@@ -20,9 +20,58 @@ MainObject *mainObj;
 
 
 int main( int argc, char **argv ) {
-//	lastResult=0.0F;
+
+	int language=LANG_EN;
+	struct stat fileStat;
+	QTranslator trans(NULL),qtTrans(NULL);
+	chdir(getenv("HOME"));
+	if(lstat(CONFIGFILE,&fileStat) == 0)
+	{
+		FILE*configFile = fopen(CONFIGFILE,"r");
+		if(configFile != NULL)
+		{
+			char* cConfFile = new char[fileStat.st_size+1];
+			fread((void*)cConfFile,fileStat.st_size,1,configFile);
+			cConfFile[fileStat.st_size]=(char)0;
+			QString confFile(cConfFile);
+
+			QString sLang=getConfigString(&confFile,"LANGUAGE");
+			if(sLang.lower()=="de")
+				language=LANG_DE;
+			else if(sLang.lower()=="fr")
+				language=LANG_FR;
+			delete[]cConfFile;
+			fclose(configFile);
+		}
+	}
+	
+	
 	QApplication a( argc, argv );
-//	a.setStyle("sgi");
+	
+	if(language==LANG_DE)
+	{
+
+
+		
+		if(trans.load("data/german.qm",INSTALLDIR))
+			a.installTranslator(&trans);
+		else MessageBox("Extcalc-Sprachpaket konnte nicht geladen werden");
+		
+		if(qtTrans.load("data/qt_de.qm",INSTALLDIR))
+			a.installTranslator(&qtTrans);
+		else MessageBox("QT-Sprachpaket konnte nicht geladen werden");
+	}
+	else if(language==LANG_FR)
+	{
+		if(trans.load("data/french.qm",INSTALLDIR))
+			a.installTranslator(&trans);
+		else MessageBox("Error loading Extcalc language file");
+		
+		if(qtTrans.load("data/qt_fr.qm",INSTALLDIR))
+			a.installTranslator(&qtTrans);
+		else MessageBox("Error loading Qt language file");
+	}
+
 	MainObject * mainObject = new MainObject;
 	mainObj=mainObject;
 
@@ -31,21 +80,8 @@ int main( int argc, char **argv ) {
 	a.setFont(stdFont);
 	setlocale(LC_NUMERIC,"C");	//use . for floating point values in every language
 	
-#ifdef LANGUAGE_DE
-	QTranslator german(NULL);
 
-	if(german.load("data/qt_de.qm",INSTALLDIR))
-		a.installTranslator(&german);
-	else MessageBox("QT-Sprachpaket konnte nicht geladen werden");
-#endif
-	
-#ifdef LANGUAGE_FR
-	QTranslator french(NULL);
 
-	if(french.load("data/qt_fr.qm",INSTALLDIR))
-		a.installTranslator(&french);
-	else MessageBox("Error loading Qt language file");
-#endif
 
 //root		8730
 //Pi		960   (982)
@@ -67,6 +103,8 @@ void MainObject::closeEvent(QCloseEvent*e)
 {
 	writeConfigFile();
 	writeVarFile();
+	writeConstants();
+	writeUIState();
 	statistics->writeListsFile();
 	if(scripting->quitProgram())
 	{
@@ -177,93 +215,213 @@ int MainObject::readConfigFile()
 		if(configFile != NULL)
 		{
 			ret=-1;
-			fwrite("ANGLE=rad\n",10,1,configFile);
-			fwrite("OUTPUTTYPE=variablenum\n",23,1,configFile);
-			fwrite("OUTPUTLENGTH=10\n",16,1,configFile);
-			fwrite("AXIS=true\n",10,1,configFile);
-			fwrite("RASTER=true\n",12,1,configFile);
-			fwrite("LABEL=false\n",12,1,configFile);
-			fwrite("AUTOSIZE=false\n",15,1,configFile);
-			fwrite("3DGRID=true\n",12,1,configFile);
-			fwrite("LOGNYQUISTSTEPS=true\n",21,1,configFile);
-			fwrite("CLEARSCRIPTMEM=true\n",20,1,configFile);
-			fwrite("XMIN=-10.0\n",11,1,configFile);
-			fwrite("XMAX=10.0\n",10,1,configFile);
-			fwrite("YMIN=-10.0\n",11,1,configFile);
-			fwrite("YMAX=10.0\n",10,1,configFile);
-			fwrite("ZMIN=-10.0\n",11,1,configFile);
-			fwrite("ZMAX=10.0\n",10,1,configFile);
-			fwrite("RASTERSIZEX=1\n",14,1,configFile);
-			fwrite("RASTERSIZEY=1\n",14,1,configFile);
-			fwrite("RASTERSIZEZ=1\n",14,1,configFile);
-			fwrite("RADIUSMAX=10\n",13,1,configFile);
-			fwrite("ANGLEMAX=360\n",13,1,configFile);
-			fwrite("RASTERSIZERADIUS=1\n",19,1,configFile);
-			fwrite("RASTERSIZEANGLE=15\n",19,1,configFile);
-			fwrite("PARAMETERSTART=0\n",17,1,configFile);
-			fwrite("PARAMETEREND=10\n",16,1,configFile);
-			fwrite("PARAMETERSTEPS=400\n",19,1,configFile);
-			fwrite("NYQUISTSTART=-3\n",16,1,configFile);
-			fwrite("NYQUISTEND=3\n",13,1,configFile);
-			fwrite("NYQUISTSTEPS=400\n",17,1,configFile);
-			fwrite("GRAPH3DSTEPS=50\n",16,1,configFile);
-			fwrite("GRAPH2DSTEPS=400\n",17,1,configFile);
-			fwrite("DYNAMICSTART=0\n",15,1,configFile);
-			fwrite("DYNAMICEND=10\n",14,1,configFile);
-			fwrite("DYNAMICSTEPS=10\n",16,1,configFile);
-			fwrite("DYNAMICDELAY=1\n",15,1,configFile);
-			fwrite("DYNAMICMOVEUPDOWN=true\n",23,1,configFile);
-			fwrite("TABLESTARTX=0\n",14,1,configFile);
-			fwrite("TABLESTARTZ=0\n",14,1,configFile);
-			fwrite("TABLEENDX=10\n",13,1,configFile);
-			fwrite("TABLEENDZ=10\n",13,1,configFile);
-			fwrite("TABLESTEPSX=11\n",15,1,configFile);
-			fwrite("TABLESTEPSZ=11\n",15,1,configFile);
-			fwrite("ANALYSEPRECISION=1\n",19,1,configFile);
-			fwrite("GRAPHTYPE=std\n",14,1,configFile);
-			fwrite("CALCTYPE=scientific\n",20,1,configFile);
-			fwrite("COMPLEX=false\n",14,1,configFile);
-			fwrite("TABLETYPE=normal\n",17,1,configFile);
-			fwrite("BASE=dec\n",9,1,configFile);
-			fwrite("STATSHOWLINES=false\n",20,1,configFile);
-			fwrite("STATSHOWPOINTS=true\n",20,1,configFile);
-			fwrite("STATAUTOCLEAR=true\n",19,1,configFile);
-			fwrite("SHOWWIN1=true\n",14,1,configFile);
-			fwrite("SHOWWIN2=false\n",15,1,configFile);
-			fwrite("SHOWWIN3=true\n",14,1,configFile);
-			fwrite("SHOWWIN4=true\n",14,1,configFile);
-			fwrite("SHOWWIN5=true\n",14,1,configFile);
-			fwrite("SHOWWIN6=false\n",15,1,configFile);
-			fwrite("SHOWWIN7=true\n",15,1,configFile);
-			fwrite("SHOWWIN8=false\n",15,1,configFile);
-			fwrite("SCRIPTROOT=",11,1,configFile);
-			fwrite(getenv("HOME"),strlen(getenv("HOME")),1,configFile);
-			fwrite("/.extcalc/script\n",17,1,configFile);
-			fwrite("CODEPATH=code\n",14,1,configFile);
-			fwrite("DATAPATH=data\n",14,1,configFile);
+			QString file("ANGLE=rad\n\
+OUTPUTTYPE=variablenum\n\
+OUTPUTLENGTH=15\n\
+XMIN=-10\n\
+XMAX=10\n\
+YMIN=-10\n\
+YMAX=10\n\
+ZMIN=-10\n\
+ZMAX=10\n\
+RADIUSMAX=10\n\
+ANGLEMAX=6.28319\n\
+RASTERSIZEX=1\n\
+RASTERSIZEY=1\n\
+RASTERSIZEZ=1\n\
+RASTERSIZEANGLE=0.314159\n\
+RASTERSIZERADIUS=1\n\
+NYQUISTSTART=-3\n\
+NYQUISTEND=3\n\
+PARAMETERSTART=-10\n\
+PARAMETEREND=10\n\
+PARAMETERSTEPS=400\n\
+DYNAMICSTART=0\n\
+DYNAMICEND=30\n\
+DYNAMICSTEPS=15\n\
+DYNAMICDELAY=10\n\
+TABLESTARTX=0\n\
+TABLEENDX=10\n\
+TABLESTARTZ=0\n\
+TABLEENDZ=10\n\
+TABLESTEPSX=11\n\
+TABLESTEPSZ=11\n\
+GRAPH2DSTEPS=400\n\
+GRAPH3DSTEPS=50\n\
+NYQUISTSTEPS=400\n\
+ANALYSEPRECISION=1\n\
+SCRIPTROOT=%1/.extcalc/script\n\
+CODEPATH=code\n\
+DATAPATH=data\n\
+RASTER=true\n\
+AXIS=true\n\
+LABEL=true\n\
+AUTOSIZE=true\n\
+DYNAMICMOVEUPDOWN=false\n\
+3DGRID=true\n\
+LOGNYQUISTSTEPS=true\n\
+GRAPHTYPE=std\n\
+CALCTYPE=scientific\n\
+COMPLEX=true\n\
+CLEARSCRIPTMEM=true\n\
+STATSHOWLINES=true\n\
+STATSHOWPOINTS=true\n\
+STATAUTOCLEAR=true\n\
+TABLETYPE=normal\n\
+BASE=hex\n\
+LANGUAGE=en\n\
+SHOWWIN1=true\n\
+SHOWWIN2=false\n\
+SHOWWIN3=true\n\
+SHOWWIN4=true\n\
+SHOWWIN5=true\n\
+SHOWWIN6=true\n\
+SHOWWIN7=true\n\
+SHOWWIN8=false\n\
+F1=sinX\n\
+F2=(X+5)^2-4\n\
+F3=if(A<=10) (1-A/5)*x^2; else if(A<=20) -(x+5*(A-10)/10)^2; else -(x+5)^2-4*(A-20)/10;\n\
+F4=\n\
+F5=X+2\n\
+F6=5*sin(4X)\n\
+F7=2/(1+0.7cosX)\n\
+F8=\n\
+F9=T\\T^2\n\
+F10=sinT*(exp(cosT)-2*cos(4T)-(sin(T/12))^5)\\cosT*(exp(cosT)-2*cos(4T)-(sin(T/12))^5)\n\
+F11=F=.01*T^3+1;U=1-A/15;((1+U)*T+(1-U)*F)/2\\((1-U)*T+(1+U)*F)/2\n\
+F12=(1-A/15)*T\\T^3-5*T^2+5*T-2\n\
+F13=F=.05*T^3;if(T>0)U=atan(F/T);elseU=0;R=sqrt(T^2+F^2);cos(pi/30*A+U)*R\\sin(pi/30*A+U)*R\n\
+F14=\n\
+F15=sin(X)*cos(Z)\n\
+F16=L=0.01;U=2;K=1e-3;if(X<U)0;else{if(Z>0 && Z<X-U)K*Z*(X-U-Z/2)*(1+L*Z);else if (Z>0) K/2*(X-U)*(1+L*Z); else 0;}\n\
+F17=abs(arg(((X+Zi)^2-1)*((X+Zi)^2-2-i)^2/((X+Zi)^2+2+2i)))\n\
+F18=\n\
+F19=exp(Z*i)\n\
+F20=10/(1+Z*i)\n\
+COMMENT1=Sine wave, Coordinate system: trigonometric standard size\n\
+COMMENT2=Parabola\n\
+COMMENT3=Animated parabola\n\
+COMMENT4=\n\
+COMMENT5=Archimedian spiral\n\
+COMMENT6=Polar rose\n\
+COMMENT7=Ellipse\n\
+COMMENT8=\n\
+COMMENT9=Parabola\n\
+COMMENT10=Butterfly curve; Coordinate system: polar, max. radius=5; T from 0 to 20\n\
+COMMENT11=Inverse function; T from -10 to 10\n\
+COMMENT12=Reflection\n\
+COMMENT13=Odd function; T from 0 to 10\n\
+COMMENT14=\n\
+COMMENT15=X, Y and Z from -3 to 3\n\
+COMMENT16=MOSFET transistor current; Coordinate system: X and Z from 0 to 10,Y from -0.01 to 0.04\n\
+COMMENT17=Complex graph in 3D (X=Re,Z=Im); Coordinate system: X and Z from -2 to 2,Y from 0 to 5; complex enabled (Calculator)\n\
+COMMENT18=\n\
+COMMENT19=\n\
+COMMENT20=\n\
+F1ACTIVE=true\n\
+F2ACTIVE=false\n\
+F3ACTIVE=false\n\
+F4ACTIVE=false\n\
+F5ACTIVE=false\n\
+F6ACTIVE=false\n\
+F7ACTIVE=false\n\
+F8ACTIVE=false\n\
+F9ACTIVE=false\n\
+F10ACTIVE=false\n\
+F11ACTIVE=false\n\
+F12ACTIVE=false\n\
+F13ACTIVE=false\n\
+F14ACTIVE=false\n\
+F15ACTIVE=false\n\
+F16ACTIVE=false\n\
+F17ACTIVE=false\n\
+F18ACTIVE=false\n\
+F19ACTIVE=false\n\
+F20ACTIVE=false\n\
+F1DYNAMIC=false\n\
+F2DYNAMIC=false\n\
+F3DYNAMIC=true\n\
+F4DYNAMIC=false\n\
+F5DYNAMIC=false\n\
+F6DYNAMIC=false\n\
+F7DYNAMIC=false\n\
+F8DYNAMIC=false\n\
+F9DYNAMIC=false\n\
+F10DYNAMIC=false\n\
+F11DYNAMIC=true\n\
+F12DYNAMIC=true\n\
+F13DYNAMIC=true\n\
+F14DYNAMIC=false\n\
+F15DYNAMIC=false\n\
+F16DYNAMIC=false\n\
+F17DYNAMIC=false\n\
+F18DYNAMIC=true\n\
+F19DYNAMIC=false\n\
+F20DYNAMIC=false\n\
+F1LOGIC=false\n\
+F2LOGIC=false\n\
+F3LOGIC=true\n\
+F4LOGIC=false\n\
+F5LOGIC=false\n\
+F6LOGIC=false\n\
+F7LOGIC=false\n\
+F8LOGIC=false\n\
+F9LOGIC=false\n\
+F10LOGIC=true\n\
+F11LOGIC=true\n\
+F12LOGIC=false\n\
+F13LOGIC=true\n\
+F14LOGIC=false\n\
+F15LOGIC=false\n\
+F16LOGIC=true\n\
+F17LOGIC=true\n\
+F18LOGIC=false\n\
+F19LOGIC=false\n\
+F20LOGIC=false\n\
+F1COLOR=red\n\
+F2COLOR=black\n\
+F3COLOR=blue\n\
+F4COLOR=black\n\
+F5COLOR=red\n\
+F6COLOR=green\n\
+F7COLOR=yellow\n\
+F8COLOR=blue\n\
+F9COLOR=green\n\
+F10COLOR=red\n\
+F11COLOR=magenta\n\
+F12COLOR=grey\n\
+F13COLOR=cyan\n\
+F14COLOR=green\n\
+F15COLOR=colored\n\
+F16COLOR=colored\n\
+F17COLOR=colored\n\
+F18COLOR=red\n\
+F19COLOR=blue\n\
+F20COLOR=green\n\
+F1TYPE=std\n\
+F2TYPE=std\n\
+F3TYPE=std\n\
+F4TYPE=std\n\
+F5TYPE=polar\n\
+F6TYPE=polar\n\
+F7TYPE=polar\n\
+F8TYPE=std\n\
+F9TYPE=parameter\n\
+F10TYPE=parameter\n\
+F11TYPE=parameter\n\
+F12TYPE=parameter\n\
+F13TYPE=parameter\n\
+F14TYPE=std\n\
+F15TYPE=3d\n\
+F16TYPE=3d\n\
+F17TYPE=3d\n\
+F18TYPE=std\n\
+F19TYPE=nyquist3d\n\
+F20TYPE=nyquist2d\n");
+			file.arg(getenv("HOME"));
 			
-			fwrite("F1=sinX\n",8,1,configFile);
-			for(int c=1; c<9; c++)
-				fwrite("F"+QString::number(c+1)+"=\n",4,1,configFile);
-			for(int c=9; c<20; c++)
-				fwrite("F"+QString::number(c+1)+"=\n",5,1,configFile);
-			fwrite("F1ACTIVE=true\n",14,1,configFile);
-			for(int c=1; c<9; c++)
-				fwrite("F"+QString::number(c+1)+"ACTIVE=false\n",15,1,configFile);
-			for(int c=9; c<20; c++)
-				fwrite("F"+QString::number(c+1)+"ACTIVE=false\n",16,1,configFile);
-			for(int c=0; c<9; c++)
-				fwrite("F"+QString::number(c+1)+"COLOR="+QString(GRAPHH_COL1)+"\n",9+strlen(GRAPHH_COL1),1,configFile);
-			for(int c=9; c<20; c++)
-				fwrite("F"+QString::number(c+1)+"COLOR="+QString(GRAPHH_COL1)+"\n",10+strlen(GRAPHH_COL1),1,configFile);
-			for(int c=0; c<9; c++)
-				fwrite("F"+QString::number(c+1)+"TYPE=std\n",11,1,configFile);
-			for(int c=9; c<20; c++)
-				fwrite("F"+QString::number(c+1)+"TYPE=std\n",12,1,configFile);
-			for(int c=0; c<9; c++)
-				fwrite("F"+QString::number(c+1)+"DYNAMIC=false\n",16,1,configFile);
-			for(int c=9; c<20; c++)
-				fwrite("F"+QString::number(c+1)+"DYNAMIC=false\n",17,1,configFile);
+			
+			fwrite(file.latin1(),file.length(),1,configFile);
 			fclose(configFile);
 			
 			if(lstat(CONFIGFILE,&fileStat) != 0)
@@ -481,6 +639,14 @@ int MainObject::readConfigFile()
 			dynDelay=1;
 		pref.dynamicDelay=dynDelay;
 	}
+	QString paraStartStr=getConfigString(&confFile,"PARAMETERSTART");
+	if(paraStartStr.length()>0)
+	{
+		double paraStart=paraStartStr.toDouble();
+		if(paraStart == NAN)
+			paraStart=0.0;
+		pref.parameterStart=paraStart;
+	}	
 	QString paraEndStr=getConfigString(&confFile,"PARAMETEREND");
 	if(paraEndStr.length()>0)
 	{
@@ -740,6 +906,16 @@ int MainObject::readConfigFile()
 		else if(tableType.lower() == "complex")
 			pref.tableType=TABLECOMPLEX;
 		
+	}
+	QString language=getConfigString(&confFile,"LANGUAGE");
+	if(language.length()>0)
+	{
+		if(language.lower() == "de")
+			pref.language=LANG_DE;
+		else if(language.lower() == "fr")
+			pref.language=LANG_FR;
+		else pref.language=LANG_EN;
+		
 	}
 	for(int c=0; c<8; c++)
 	{
@@ -754,6 +930,7 @@ int MainObject::readConfigFile()
 	for(int c=0; c<20; c++)
 	{
 		functionStrings[c]=getConfigString(&confFile,"F"+QString::number(c+1));
+		functionStrings[c]=resetConfigString(functionStrings[c]);
 		if(functionStrings[c].length()<=0)
 			functionStrings[c]="";
 	}
@@ -761,6 +938,18 @@ int MainObject::readConfigFile()
 		delete[]pref.functions;
 	pref.functions=functionStrings;
 	
+	QString*commentStrings=new QString[20];
+	
+	for(int c=0; c<20; c++)
+	{
+		commentStrings[c]=getConfigString(&confFile,"COMMENT"+QString::number(c+1));
+		if(commentStrings[c].length()<=0)
+			commentStrings[c]="";
+	}
+	if(pref.functionComments!=NULL)
+		delete[]pref.functionComments;
+	pref.functionComments=commentStrings;
+	
 	bool*activeFunctions=new bool[20];
 	
 	for(int c=0; c<20; c++)
@@ -785,6 +974,18 @@ int MainObject::readConfigFile()
 	if(pref.dynamicFunctions!=NULL)
 		delete[]pref.dynamicFunctions;
 	pref.dynamicFunctions=dynamicFunctions;
+	
+	bool*logicFunctions=new bool[20];
+	for(int c=0; c<20; c++)
+	{
+		QString logicString=getConfigString(&confFile,"F"+QString::number(c+1)+"LOGIC");
+		if(logicString.lower() =="true" &&logicString.length()>0)
+			logicFunctions[c]=true;
+		else logicFunctions[c]=false;
+	}
+	if(pref.logicFunctions!=NULL)
+		delete[]pref.logicFunctions;
+	pref.logicFunctions=logicFunctions;
 
 	QString scriptPath=getConfigString(&confFile,"SCRIPTROOT");
 	if(scriptPath.length()<=0)
@@ -1030,7 +1231,13 @@ void MainObject::writeConfigFile()
 		configuration+="bin";
 	else if(pref.base == DEC)
 		configuration+="dec";
-	configuration+="\n";
+	configuration+="\nLANGUAGE=";
+	if(pref.language == LANG_DE)
+		configuration+="de\n";
+	else if(pref.language == LANG_FR)
+		configuration+="fr\n";
+	else configuration+="en\n";
+
 
 	for(int c=0; c<8;c++)
 	{
@@ -1047,6 +1254,10 @@ void MainObject::writeConfigFile()
 		configuration+=cleanConfigString("F"+QString::number(c+1),pref.functions[c]);
 	}
 	for(int c=0; c<20;c++)
+	{
+		configuration+="COMMENT"+QString::number(c+1)+"="+pref.functionComments[c]+"\n";
+	}
+	for(int c=0; c<20;c++)
 	{
 		configuration+="F";
 		configuration+=QString::number(c+1);
@@ -1065,6 +1276,15 @@ void MainObject::writeConfigFile()
 		else configuration+="false\n";
 	}
 	for(int c=0; c<20;c++)
+	{
+		configuration+="F";
+		configuration+=QString::number(c+1);
+		configuration+="LOGIC=";
+		if(pref.logicFunctions[c])
+			configuration+="true\n";
+		else configuration+="false\n";
+	}
+	for(int c=0; c<20;c++)
 	{
 		configuration+="F";
 		configuration+=QString::number(c+1);
@@ -1267,6 +1487,230 @@ void MainObject::writeVarFile()
 	fclose(configFile);
 }
 
+
+
+void MainObject::initConstants()
+{
+	if(pref.constList!=NULL)
+	{
+		for(int c=0; c<pref.constLen; c++)
+		{
+			delete pref.constList[c].description;
+			delete pref.constList[c].identifier;
+			delete pref.constList[c].value;
+		}
+			
+		free(pref.constList);
+	}
+	pref.constList=(Constant*)malloc(31*sizeof(Constant));
+	pref.constLen=31;
+	pref.userConstLen=0;
+
+	
+	pref.constList[0].identifier=new QString("c_pi");
+	pref.constList[0].description=new QString(tr("pi ")+getUnicode(PISTRING));
+	pref.constList[0].value=new QString("3.14159265358979323846264338327950288");
+	
+	pref.constList[1].identifier=new QString("c_eu");
+	pref.constList[1].description=new QString(tr("euler number e"));
+	pref.constList[1].value=new QString("2.71828182845904523536028747135266249");
+	
+	pref.constList[2].identifier=new QString("c_C");
+	pref.constList[2].description=new QString(tr("euler constant C"));
+	pref.constList[2].value=new QString("0.57721566490153286060651209008240243");
+
+	
+	pref.constList[3].identifier=new QString("c_c0");
+	pref.constList[3].description=new QString(tr("vacuum light speed c0 [m/s]"));
+	pref.constList[3].value=new QString("299792458");
+		
+	pref.constList[4].identifier=new QString("c_g");
+	pref.constList[4].description=new QString(tr("standard acceleration of gravity g [m/s^2]"));
+	pref.constList[4].value=new QString("9.80665");
+			
+	pref.constList[5].identifier=new QString("c_G");
+	pref.constList[5].description=new QString(tr("Newtonian constant of gravitation G [m^3/(kg s^2)]"));
+	pref.constList[5].value=new QString("6.674 28e-11");	
+	
+	pref.constList[6].identifier=new QString("c_e0");
+	pref.constList[6].description=new QString(tr("electric field constant e0 [F/m]"));
+	pref.constList[6].value=new QString("8.854187817e-12");
+	
+	pref.constList[7].identifier=new QString("c_my0");
+	pref.constList[7].description=new QString(tr("magnetic field constant my0 [N/A^2]"));
+	pref.constList[7].value=new QString("12.566370614e-7");
+
+	pref.constList[8].identifier=new QString("c_hj");
+	pref.constList[8].description=new QString(tr("Planck constant h [Js]"));
+	pref.constList[8].value=new QString("6.62606896e-34");
+	
+	pref.constList[9].identifier=new QString("c_hev");
+	pref.constList[9].description=new QString(tr("Planck constant h [eVs]"));
+	pref.constList[9].value=new QString("4.13566733e-15");
+
+	pref.constList[10].identifier=new QString("c_Z0");
+	pref.constList[10].description=new QString(tr("characteristic impedance of vacuum Z0 [V/A]"));
+	pref.constList[10].value=new QString("376.730313461");
+	
+	pref.constList[11].identifier=new QString("c_F");
+	pref.constList[11].description=new QString(tr("Faraday constant F [C/mol]"));
+	pref.constList[11].value=new QString("96485.3399");	
+	
+	pref.constList[12].identifier=new QString("c_e");
+	pref.constList[12].description=new QString(tr("atomic unit of charge e [C]"));
+	pref.constList[12].value=new QString("1.602176487e-19");
+	
+	pref.constList[13].identifier=new QString("c_alpha");
+	pref.constList[13].description=new QString(tr("fine-structure constant alpha"));
+	pref.constList[13].value=new QString("7.2973525376e-3");
+
+	pref.constList[14].identifier=new QString("c_mu");
+	pref.constList[14].description=new QString(tr("atomic mass constant mu [kg]"));
+	pref.constList[14].value=new QString("1.660 538 782 e-27");	
+
+	pref.constList[15].identifier=new QString("c_me");
+	pref.constList[15].description=new QString(tr("electron mass me [kg]"));
+	pref.constList[15].value=new QString("9.10938215e-31");
+
+	pref.constList[16].identifier=new QString("c_mn");
+	pref.constList[16].description=new QString(tr("neutron mass mn [kg]"));
+	pref.constList[16].value=new QString("1.674927211e-27");
+
+	pref.constList[17].identifier=new QString("c_mp");
+	pref.constList[17].description=new QString(tr("proton mass mp [kg]"));
+	pref.constList[17].value=new QString("1.672621637e-27");
+	
+	pref.constList[18].identifier=new QString("c_NA");
+	pref.constList[18].description=new QString(tr("Avogadro constant [1/mol]"));
+	pref.constList[18].value=new QString("6.02214179 e23");	
+	
+	pref.constList[19].identifier=new QString("c_k");
+	pref.constList[19].description=new QString(tr("Boltzmann constant k [J/K]"));
+	pref.constList[19].value=new QString("1.3806504e-23");	
+	
+	pref.constList[20].identifier=new QString("c_R0");
+	pref.constList[20].description=new QString(tr("molar gas constant R0 [J/(mol K)]"));
+	pref.constList[20].value=new QString("8.314472");	
+	
+	pref.constList[21].identifier=new QString("c_sigma");
+	pref.constList[21].description=new QString(tr("Stefan-Boltzmann constant sigma [W/(M^2 K^4)]"));
+	pref.constList[21].value=new QString("5.670400e-8");	
+	
+	pref.constList[22].identifier=new QString("c_b");
+	pref.constList[22].description=new QString(tr("Wien wavelength displacement law constant b [m K]"));
+	pref.constList[22].value=new QString("2.8977685e-3");	
+
+
+	
+	pref.constList[23].identifier=new QString("c_kmmile");
+	pref.constList[23].description=new QString(tr("km -> mile"));
+	pref.constList[23].value=new QString("0.621371192");
+	
+	pref.constList[24].identifier=new QString("c_milekm");
+	pref.constList[24].description=new QString(tr("mile -> km"));
+	pref.constList[24].value=new QString("1.609334");
+	
+	pref.constList[25].identifier=new QString("c_cminch");
+	pref.constList[25].description=new QString(tr("cm -> inch"));
+	pref.constList[25].value=new QString("0.393700787");
+	
+	pref.constList[26].identifier=new QString("c_inchcm");
+	pref.constList[26].description=new QString(tr("inch -> cm"));
+	pref.constList[26].value=new QString("2.54");
+	
+	pref.constList[27].identifier=new QString("c_goz");
+	pref.constList[27].description=new QString(tr("g -> oz"));
+	pref.constList[27].value=new QString("0.035273965");
+	
+	pref.constList[28].identifier=new QString("c_ozg");
+	pref.constList[28].description=new QString(tr("oz -> g"));
+	pref.constList[28].value=new QString("28.34962");
+		
+	pref.constList[29].identifier=new QString("c_Jcal");
+	pref.constList[29].description=new QString(tr("J -> calorie"));
+	pref.constList[29].value=new QString("0.238902957");
+	
+	pref.constList[30].identifier=new QString("c_calJ");
+	pref.constList[30].description=new QString(tr("calorie -> J"));
+	pref.constList[30].value=new QString("4.1858");
+	
+	
+	FILE*configFile;
+
+	chdir(getenv("HOME"));
+	
+	int fileLen;
+	struct stat fileStat;
+
+	if(lstat(CONSTFILE,&fileStat) != 0)
+	{
+		getPref(pref);
+		return;
+	}
+	else fileLen=fileStat.st_size;
+	
+	configFile = fopen(CONSTFILE,"r");
+	if(configFile == NULL)
+	{
+		ErrorBox(tr("Can't read constants file %1.\n\n").arg(CONSTFILE)+getErrorMessage());
+		getPref(pref);
+		return;
+	}
+	char*buffer=new char[fileLen+1];
+	buffer[fileLen]=(char)0;
+	fread((void*)buffer,fileLen,1,configFile);
+	QString constData(buffer),desc,val;
+	int pos1=0,pos2=0;
+	
+	int c=1;
+	while((pos1=constData.find("\n",pos2))!=-1)
+	{
+		desc=constData.mid(pos2,pos1-pos2);
+		pos2=pos1+1;
+		pos1=constData.find("\n",pos2);
+		if(pos1==-1)
+			break;
+		val=constData.mid(pos2,pos1-pos2);
+		pos2=pos1+1;
+		
+		pref.constLen++;
+		pref.userConstLen++;
+		pref.constList=(Constant*)realloc((void*)pref.constList,sizeof(Constant)*pref.constLen);
+		pref.constList[pref.constLen-1].description=new QString(desc);
+		pref.constList[pref.constLen-1].value=new QString(val);
+		pref.constList[pref.constLen-1].identifier=new QString("c_usr"+QString::number(c));
+		c++;
+	}
+	fclose(configFile);
+	delete[] buffer;
+
+	getPref(pref);
+}
+
+void MainObject::writeConstants()
+{
+	FILE*configFile;
+
+	chdir(getenv("HOME"));
+	configFile = fopen(CONSTFILE,"w");
+	if(configFile == NULL)
+	{
+		ErrorBox(tr("Constants file %1 could not be written.\n\n").arg(CONSTFILE)+getErrorMessage());
+		return;
+	}
+	QString constData("");
+	for(int c=pref.constLen-pref.userConstLen; c<pref.constLen; c++)
+	{
+		constData+=*pref.constList[c].description;
+		constData+="\n";
+		constData+=*pref.constList[c].value;
+		constData+="\n";
+	}
+	
+	fwrite(constData.latin1(),constData.length(),1,configFile);
+	fclose(configFile);
+}
+
 void MainObject::tabChangeSlot(QWidget*activeWidget)
 {
 	calculator->dockWindowSlot();
@@ -1402,18 +1846,29 @@ void MainObject::helpMenuSlot(int item)
 	switch(item)
 	{
 	case EXTHELP:
+		if(helpBrowser!=NULL)
+			delete helpBrowser;
+		helpBrowser=new HelpBrowser(this);
+		helpBrowser->setGeometry(50,50,750,550);
+
+		if(pref.language==LANG_DE)
+			helpBrowser->setContent(QString(INSTALLDIR)+"/doc/help_de.html");
+		else if(pref.language==LANG_FR)
+			helpBrowser->setContent(QString(INSTALLDIR)+"/doc/help_fr.html");
+		else helpBrowser->setContent(QString(INSTALLDIR)+"/doc/help_en.html");
+		helpBrowser->show();
+			
+		/*
 		helpProcess->clearArguments();
 		helpProcess->addArgument("konqueror");
-#ifdef LANGUAGE_DE
-		helpProcess->addArgument(QString(INSTALLDIR)+"/doc/help_de.html");
-#endif
-#ifdef LANGUAGE_EN
-		helpProcess->addArgument(QString(INSTALLDIR)+"/doc/help_en.html");
-#endif
-#ifdef LANGUAGE_FR
-		helpProcess->addArgument(QString(INSTALLDIR)+"/doc/help_fr.html");
-#endif
+		if(pref.language==LANG_DE)
+			helpProcess->addArgument(QString(INSTALLDIR)+"/doc/help_de.html");
+		else if(pref.language==LANG_FR)
+			helpProcess->addArgument(QString(INSTALLDIR)+"/doc/help_fr.html");
+		else helpProcess->addArgument(QString(INSTALLDIR)+"/doc/help_en.html");
+
 		helpProcess->start();
+		*/
 		break;
 	case INFO:
 		infoDialog->exec();
@@ -1512,6 +1967,27 @@ void MainObject::graphTypeMenuSlot(int item)
 	pref.graphType=item;
 	getPref(pref);
 }
+void MainObject::graphMenuSlot(int item)
+{
+	if(item==GRAPHIMPORT)
+	{
+		if(functionDialog!=NULL)
+		{
+			QObject::disconnect(SIGNAL(prefChange(Preferences)),this);
+			delete functionDialog;
+		}
+		functionDialog=new ImportDialog(pref,this,true,true);
+		QObject::connect(functionDialog,SIGNAL(prefChange(Preferences)),this,SLOT(getPref(Preferences)));
+		functionDialog->show();
+	}
+	else if(item==GRAPHEXPORT)
+	{
+		if(functionDialog!=NULL)
+			delete functionDialog;
+		functionDialog=new ImportDialog(pref,this,false,true);	
+		functionDialog->show();
+	}
+}
 
 void MainObject::tableMenuSlot(int item)
 {
@@ -1628,11 +2104,25 @@ void MainObject::prefMenuSlot(int item)
 	}
 }
 
+void MainObject::languageMenuSlot(int item)
+{
+	pref.language=item;
+	getPref(pref);
+	int ret=YesNoCancelBox(tr("Extcalc must be restarted to apply this changes!\n\nRestart now?"));
+	if(ret==0)
+	{
+		QProcess extcalcProcess;
+		extcalcProcess.addArgument("extcalc");
+		extcalcProcess.start();
+		qApp->closeAllWindows();
+	}
+}
+
 void MainObject::fileMenuSlot(int item)
 {
 	
 	if(item==QUIT)
-		qApp->quit();
+		qApp->closeAllWindows();
 }
 
 void MainObject::editMenuSlot(int item)
@@ -1807,86 +2297,116 @@ void MainObject::changeTabSlot(int num)
 	}
 }
 
-
-
-
-void ImportDialog::windowActivationChange(bool)
+void MainObject::writeUIState()
 {
-	if(!isActiveWindow()&&!dialog)
-		setActiveWindow();
+	FILE*configFile;
+	chdir(getenv("HOME"));
+	configFile = fopen(UIFILE,"w");
+	if(configFile == NULL)
+		return;
+	//0             1              2..7
+	//window width, window height, function table column 1-6
+	int buffer[8];
+	buffer[0]=width();
+	buffer[1]=height();
+	graph->getUIState(&buffer[2]);
+	
+	fwrite(buffer,8*sizeof(int),1,configFile);
 
+	fclose(configFile);
 }
 
-void ImportDialog::saveSlot()
+void MainObject::readUIState()
 {
-	QString savePath=savePathLine->text(),openPath=openPathLine->text();
+	chdir(getenv("HOME"));
+	int fileLen;
+	struct stat fileStat;
+
+	if(lstat(UIFILE,&fileStat) != 0)
+		return;
+	else fileLen=fileStat.st_size;
 	
-	if(savePath.find("/") <0 && importFile)
-		savePath=pref.scriptPath+"/"+pref.scriptDirName+"/"+savePath;
+	if(fileLen!=8*sizeof(int))
+		return;
 	
-	if(openPath.find("/") <0 && !importFile)
-		openPath=pref.scriptPath+"/"+pref.scriptDirName+"/"+openPath;
+	FILE*configFile = fopen(UIFILE,"r");
+	if(configFile == NULL)
+		return;
 	
-	if((savePath.find(pref.scriptPath+"/"+pref.scriptDirName) !=0 && importFile) || 
-		  (openPath.find(pref.scriptPath+"/"+pref.scriptDirName) !=0 && !importFile))
-	{
-		ErrorBox(EXTCALCC_MSG6+savePath);
-	}
-	else
-	{
-		struct stat testStat;
-		
-		if(lstat(savePath,&testStat)==0)
-		{
-			if(YesNoBox(EXTCALCC_MSG8+savePath)==1)
-				return;
-		}
-		system("cp -f "+openPath+" "+savePath);
-		emit updateScriptSignal(FILEUPDATE);
-		close();
-	}
+	
+	int buffer[8];
+	fread(buffer,8*sizeof(int),1,configFile);
+	
+	resize(buffer[0],buffer[1]);
+	graph->setUIState(&buffer[2]);
+
+	fclose(configFile);
 }
 
 
-void ImportDialog::openDialogSlot()
+
+
+HelpBrowser::HelpBrowser(QWidget*parent) :QWidget(parent,"Help Browser",Qt::WType_TopLevel)
 {
-	QString startPath,dialogText;
-	dialog=true;
-	if(!importFile) {
-		startPath=pref.scriptPath+"/"+pref.scriptDirName;
-		dialogText=QString(EXTCALCC_MSG9);
-	}
-	else {
-		dialogText=QString(EXTCALCC_MSG10);
-		startPath=QString(getenv("HOME"));
-	}
+	currentSource="";
+	toolBar=new QToolBar();
+	dockArea=new QDockArea(Qt::Horizontal,QDockArea::Normal,this);
+	dockArea->moveDockWindow(toolBar);
+	toolBar->setMovingEnabled(false);
 	
-	QString path = QFileDialog::getOpenFileName(
-			startPath,
-	QString::null,
-	this,
-	EXTCALCC_MSG11,
-	dialogText);
-	dialog=false;
-	if(!path.isNull())
-		openPathLine->setText(path);
-}
+	forwardIcon=new QPixmap(INSTALLDIR+QString("/data/forward.png"));
+	backIcon=new QPixmap(INSTALLDIR+QString("/data/back.png"));
+	zoominIcon=new QPixmap(INSTALLDIR+QString("/data/zoomin.png"));
+	zoomoutIcon=new QPixmap(INSTALLDIR+QString("/data/zoomout.png"));
+	
+	browser=new QTextBrowser(this);
+	
+	backButton=new QToolButton(*backIcon,"","",browser,SLOT(backward()),toolBar);	
+	forwardButton=new QToolButton(*forwardIcon,"","",browser,SLOT(forward()),toolBar);
+	zoominButton=new QToolButton(*zoominIcon,"","",this,SLOT(zoominSlot()),toolBar);
+	zoomoutButton=new QToolButton(*zoomoutIcon,"","",this,SLOT(zoomoutSlot()),toolBar);
+	
+	forwardButton->setOn(false);
+	backButton->setOn(false);
 
-void ImportDialog::saveDialogSlot()
+	connect(browser,SIGNAL(forwardAvailable(bool)),forwardButton,SLOT(setOn(bool)));
+	connect(browser,SIGNAL(backwardAvailable(bool)),backButton,SLOT(setOn(bool)));
+	connect(browser,SIGNAL(sourceChanged(const QString&)),this,SLOT(sourceSlot(const QString&)));
+
+	
+}
+void HelpBrowser::setContent(QString path)
 {
-	dialog=true;
-	QString startPath;
-	if(importFile)
-		startPath=pref.scriptPath+"/"+pref.scriptDirName;
-	else startPath=QString(getenv("HOME"));
-	QString path = QFileDialog::getSaveFileName(
-			startPath,
-	QString::null,
-	this,
-	EXTCALCC_MSG12,
-	EXTCALCC_MSG13 );
-	dialog=false;
-	if(!path.isNull())
-		savePathLine->setText(path);
+	browser->setSource(path);
+	currentSource=path;
 }
 
+void HelpBrowser::zoominSlot()
+{
+	browser->zoomIn(2);
+	browser->repaint();
+}
+
+void HelpBrowser::zoomoutSlot()
+{
+		browser->zoomOut(2);
+		browser->repaint();
+
+}
+
+void HelpBrowser::sourceSlot(const QString &source)
+{
+	if(currentSource!="" && source.find(currentSource)!=0)
+		browser->setSource(currentSource);
+}
+
+void HelpBrowser::resizeEvent(QResizeEvent*)
+{
+	dockArea->setGeometry(0,0,width(),35);
+	browser->setGeometry(0,35,width(),height()-35);
+}
+
+
+
+
+
