@@ -32,6 +32,7 @@ dialog, the todo list and the bug list.
 #include "matrixwidget.h"
 #include "statistics.h"
 #include "global.h"
+#include "importdialog.h"
 #include <qtabwidget.h>
 #include <qtabbar.h>
 #include <qstring.h>
@@ -41,6 +42,11 @@ dialog, the todo list and the bug list.
 #include <float.h>
 #include <qtranslator.h>
 #include <locale.h>
+#include <qtextbrowser.h>
+#include <qtoolbar.h>
+#include <qdockarea.h>
+#include <qiconset.h>
+#include <qtoolbutton.h>
 
 
 //Unicode characters:
@@ -137,12 +143,18 @@ dialog, the todo list and the bug list.
 //  - optimization of the split and parse methods                                           //
 //  - change structure of exec and calc methods                                             //
 //  - standardized user interface design for all tabs                                   ok  //
-//  - open help file in default browser                                                     //
-//  - integrate constants list                                                              //
+//  - open help file in default browser                                                 ok  //
+//  - mathematic constants list                                                         ok  //
 //  - precise error messages                                                            ok  //
 //  - RPN                                                                                   //
 //  - Port to QT 4                                                                          //
-//  - use QTranslator for internationalization                                              //
+//  - use QTranslator for internationalization                                          ok  //
+//  - symbolic calculation for systems of linear equations                                  //
+//  - logic operations for graph drawing                                                ok  //
+//  - graph analysis for functions with logic operations                                    //
+//  - multi-line edit for functions in graphics window                                      //
+//  - usage of multithreading for graph calculation                                         //
+//  - import/export function for graphs                                                     //
 //                                                                                          //
 ////////////////////////////////////////beta releases/////////////////////////////////////////
 
@@ -182,7 +194,7 @@ dialog, the todo list and the bug list.
 //	- array memory can't be deleted														ok	//
 //	- script load balancing doesn't work on fast (dual-core) CPUs						ok	//
 //	- inserting text into a running script doesn't work									ok	//
-//	- gl script programming does not work on every system									//
+//	- gl script programming does not work on every system								ok	//
 //	- import script dialog path check is wrong											ok	//
 //	- bug in script console at selection												ok	//
 //	- file dialog in script preferences window is always in background					ok	//
@@ -190,6 +202,7 @@ dialog, the todo list and the bug list.
 //	- directory creation at first start does not work									ok	//
 //	- command priority in script interpreter wrong										ok	//
 //	- autosize setting leads to wrong screenshot size									ok	//
+//	- abs does not exist in standard calculator class									ok	//
 
 
 //////////////////////////used variables//////////////////////////
@@ -259,13 +272,14 @@ dialog, the todo list and the bug list.
 ////////////////////////////////////////////////////////////////////
 
 class ImportDialog;
+class HelpBrowser;
 
 class MainObject :public QTabWidget
 {
 	QMenuBar*mainMenu;
 	QPopupMenu *calcMenu,*angleMenu,*helpMenu,*outputMenu,*floatPointMenu,*prefMenu,*graphMenu;
 	QPopupMenu *coordinateMenu,*graphTypeMenu,*calcTypeMenu,*baseMenu,*tableMenu,*tableTypeMenu;
-	QPopupMenu *editMenu,*viewMenu,*fileMenu,*scriptMenu,*statisticsMenu;
+	QPopupMenu *editMenu,*viewMenu,*fileMenu,*scriptMenu,*statisticsMenu,*languageMenu;
 	QTabBar*tabBar;
 	CalcWidget *calculator,*calculator2;
 	GraphWidget * graph;
@@ -275,6 +289,7 @@ class MainObject :public QTabWidget
 	ScriptPreferences*scriptPref;
 	ImportDialog*importDialog;
 	ImportDialog*exportDialog;
+	ImportDialog*functionDialog;
 	TableWidget*table;
 	ScriptWidget*scripting;
 	ScriptIOWidget*scriptIO;
@@ -287,6 +302,7 @@ class MainObject :public QTabWidget
 	QLabel*authorInfo;
 	QLabel*versionInfo;
 	QPixmap*appIcon;
+	HelpBrowser *helpBrowser;
 	Variable *vars;
 	Vector *vecs;
 	bool calcFocus;
@@ -297,7 +313,7 @@ Q_OBJECT
 public:
 MainObject() :QTabWidget()
 {
-
+	helpBrowser=NULL;
 	vars=new Variable [27];
 	for(int c=0; c<27;c++)
 		vars[c]=0.0;
@@ -364,6 +380,7 @@ MainObject() :QTabWidget()
 	scriptPref=NULL;
 	importDialog=NULL;
 	exportDialog=NULL;
+	functionDialog=NULL;
 	
 	tabBar = new QTabBar(this);
 	
@@ -391,6 +408,7 @@ MainObject() :QTabWidget()
 	pref.complex=false;
 	pref.clearScriptMemory=true;
 	pref.functions=NULL;
+	pref.functionComments=NULL;
 	pref.activeFunctions=new bool[20];
 	for(int c=0; c<20;c++)
 		pref.activeFunctions[c]=false;
@@ -403,6 +421,9 @@ MainObject() :QTabWidget()
 	pref.dynamicFunctions=new bool[20];
 	for(int c=0; c<20; c++)
 		pref.dynamicFunctions[c]=false;
+	pref.logicFunctions=new bool[20];
+	for(int c=0; c<20; c++)
+		pref.logicFunctions[c]=false;
 	pref.xmin=pref.ymin=pref.zmin=-10.0;
 	pref.xmax=pref.ymax=pref.zmax=10.0;
 	pref.rasterSizeX=pref.rasterSizeY=pref.rasterSizeZ=1.0;
@@ -439,6 +460,9 @@ MainObject() :QTabWidget()
 	pref.showStatPoints=true;
 	pref.showWindows[0]=pref.showWindows[2]=pref.showWindows[3]=pref.showWindows[4]=pref.showWindows[6]=true;
 	pref.showWindows[1]=pref.showWindows[5]=pref.showWindows[7]=false;
+	pref.language=LANG_EN;
+	pref.constList=NULL;
+	pref.constLen=pref.userConstLen=0;
 
 
 	threadData=new ThreadSync;
@@ -526,6 +550,10 @@ MainObject() :QTabWidget()
 	graphMenu=new QPopupMenu;
 	graphMenu->insertItem(EXTCALCH_MENU25,coordinateMenu,COORDINATE);
 	graphMenu->insertItem(EXTCALCH_MENU26,graphTypeMenu,GRAPHTYPE);
+	graphMenu->insertSeparator();
+	graphMenu->insertItem(tr("Import Graphs"),GRAPHIMPORT);
+	graphMenu->insertItem(tr("Export Graphs"),GRAPHEXPORT);
+	QObject::connect(graphMenu,SIGNAL(activated(int)),this,SLOT(graphMenuSlot(int)));
 	
 	tableTypeMenu=new QPopupMenu;
 	tableTypeMenu->insertItem(EXTCALCH_MENU38,TABLENORMAL);
@@ -559,11 +587,20 @@ MainObject() :QTabWidget()
 	statisticsMenu->insertItem(EXTCALCH_MENU72,STATLINES);
 	QObject::connect(statisticsMenu,SIGNAL(activated(int)),this,SLOT(statisticsMenuSlot(int)));
 	
+	languageMenu=new QPopupMenu;
+	languageMenu->insertItem(tr("English"),LANG_EN);
+	languageMenu->insertItem(tr("French"),LANG_FR);
+	languageMenu->insertItem(tr("German"),LANG_DE);
+	QObject::connect(languageMenu,SIGNAL(activated(int)),this,SLOT(languageMenuSlot(int)));
+
+	
 	prefMenu=new QPopupMenu;
 	prefMenu->insertItem(EXTCALCH_MENU10,CPREF);
 	prefMenu->insertItem(EXTCALCH_MENU11,GPREF);
 	prefMenu->insertItem(EXTCALCH_MENU45,TPREF);
 	prefMenu->insertItem(EXTCALCH_MENU60,SPREF);
+	prefMenu->insertSeparator();
+	prefMenu->insertItem(tr("Language"),languageMenu,LPREF);
 	
 	helpMenu=new QPopupMenu;
 	helpMenu->insertItem(EXTCALCH_MENU12,EXTHELP);
@@ -645,6 +682,7 @@ MainObject() :QTabWidget()
 	QObject::connect(scriptIO,SIGNAL(prefChange(Preferences)),this,SLOT(getPref(Preferences)));
 	QObject::connect(matrix,SIGNAL(prefChange(Preferences)),this,SLOT(getPref(Preferences)));
 	QObject::connect(statistics,SIGNAL(prefChange(Preferences)),this,SLOT(getPref(Preferences)));
+
 //	QObject::connect(statistics,SIGNAL(prefChange(Preferences)),this,SLOT(getPref(Preferences)));
 	QObject::connect(this,SIGNAL(currentChanged(QWidget*)),this,SLOT(tabChangeSlot(QWidget*)));
 	QObject::connect(this,SIGNAL(editSignal(int)),calculator,SLOT(editSlot(int)));
@@ -706,7 +744,9 @@ MainObject() :QTabWidget()
 			MessageBox(EXTCALCH_MENU77);
 		}
 	}
+	initConstants();
 	readVarFile();
+	readUIState();
 
 
 }
@@ -719,6 +759,10 @@ int readConfigFile();
 void readVarFile();
 void writeConfigFile();
 void writeVarFile();
+void initConstants();
+void writeConstants();
+void writeUIState();
+void readUIState();
 
 
 
@@ -736,11 +780,13 @@ void tabChangeSlot(QWidget*);
 void outputMenuSlot(int item);
 void coordinateMenuSlot(int item);
 void graphTypeMenuSlot(int item);
+void graphMenuSlot(int item);
 void floatPointMenuSlot(int item);
 void calcTypeMenuSlot(int item);
 void baseMenuSlot(int item);
 void helpMenuSlot(int item);
 void prefMenuSlot(int item);
+void languageMenuSlot(int item);
 void tableMenuSlot(int item);
 void scriptMenuSlot(int item);
 void statisticsMenuSlot(int item);
@@ -860,6 +906,10 @@ void getPref(Preferences newPref)
 	baseMenu->setItemChecked(OCT,false);
 	baseMenu->setItemChecked(DEC,false);
 	baseMenu->setItemChecked(HEX,false);
+	
+	languageMenu->setItemChecked(LANG_DE,false);
+	languageMenu->setItemChecked(LANG_EN,false);
+	languageMenu->setItemChecked(LANG_FR,false);
 
 	tableTypeMenu->setItemChecked(TABLENORMAL,false);
 	tableTypeMenu->setItemChecked(TABLEPARAMETER,false);
@@ -899,6 +949,7 @@ void getPref(Preferences newPref)
 	coordinateMenu->setItemChecked(SHOWLABELS,pref.label);
 	coordinateMenu->setItemChecked(SHOWRASTER,pref.raster);
 	coordinateMenu->setItemChecked(CONSTRATIO,pref.autosize);
+	languageMenu->setItemChecked(pref.language,true);
 	viewMenu->setItemChecked(VIEWCALC1,pref.showWindows[0]);
 	viewMenu->setItemChecked(VIEWCALC2,pref.showWindows[1]);
 	viewMenu->setItemChecked(VIEWGRAPH,pref.showWindows[2]);
@@ -919,77 +970,28 @@ signals:
 
 
 
-class ImportDialog :public QWidget
+
+class HelpBrowser :public QWidget
 {
-	QPushButton*saveButton,*cancelButton,*openDialogButton,*saveDialogButton;
-	QLabel *mainLabel,*openPathLabel,*savePathLabel;
-	QLineEdit *openPathLine,*savePathLine;
-	Preferences pref;
-	bool dialog;
-	bool importFile;
-	
-	
+	QToolBar*toolBar;
+	QDockArea*dockArea;
+	QPixmap *forwardIcon,*backIcon,*zoominIcon,*zoomoutIcon;
+	QToolButton *forwardButton,*backButton,*zoominButton,*zoomoutButton;
+	QTextBrowser *browser;
+	QString currentSource;
 	Q_OBJECT
+	
 	public:
-		ImportDialog(Preferences p,QWidget*parent,bool importF) 
-	:QWidget(parent,EXTCALCH_STR14,Qt::WStyle_Dialog | Qt::WType_Dialog)
-		{
-			pref=p;
-			dialog=false;
-			importFile=importF;
-			
-			if(importFile)
-			{
-				mainLabel=new QLabel(EXTCALCH_STR15,this);
-				openPathLabel=new QLabel(EXTCALCH_STR16,this);
-				savePathLabel=new QLabel(EXTCALCH_STR17,this);
-			}
-			else {
-				mainLabel=new QLabel(EXTCALCH_STR18,this);
-				openPathLabel=new QLabel(EXTCALCH_STR19,this);
-				savePathLabel=new QLabel(EXTCALCH_STR20,this);
-			}
-			openPathLine=new QLineEdit(this);
-			savePathLine=new QLineEdit(this);
-
-			saveButton=new QPushButton(SCRIPTPREFH_STR5,this);
-			cancelButton=new QPushButton(SCRIPTPREFH_STR6,this);
-			openDialogButton=new QPushButton(SCRIPTPREFH_STR7,this);
-			saveDialogButton=new QPushButton(SCRIPTPREFH_STR7,this);
-
-			setGeometry(0,0,410,260);
-			setFixedWidth(410);
-			setFixedHeight(260);
-		
-			mainLabel->setGeometry(20,10,380,20);
-			openPathLabel->setGeometry(20,60,380,20);
-			savePathLabel->setGeometry(20,130,380,20);
-		
-			openPathLine->setGeometry(20,90,280,20);
-			savePathLine->setGeometry(20,160,280,20);
-		
-			saveButton->setGeometry(40,210,100,30);
-			cancelButton->setGeometry(180,210,100,30);
-			openDialogButton->setGeometry(310,90,80,20);
-			saveDialogButton->setGeometry(310,160,80,20);
-		
-			QObject::connect(saveButton,SIGNAL(clicked()),this,SLOT(saveSlot()));
-			QObject::connect(openDialogButton,SIGNAL(clicked()),this,SLOT(openDialogSlot()));
-			QObject::connect(saveDialogButton,SIGNAL(clicked()),this,SLOT(saveDialogSlot()));
-			QObject::connect(cancelButton,SIGNAL(clicked()),this,SLOT(close()));
-		}
+	HelpBrowser(QWidget*parent);
+	void setContent(QString);
+	
+	public slots:
+	void zoominSlot();
+	void zoomoutSlot();
+	void sourceSlot(const QString&);
 	
 	protected:
-		virtual void windowActivationChange(bool);
-
-	public slots:
-
-		void saveSlot();
-		void openDialogSlot();
-		void saveDialogSlot();
-		
-	signals:
-		void updateScriptSignal(int);
+	void resizeEvent(QResizeEvent*);
 };
 
 
@@ -999,6 +1001,4 @@ class ImportDialog :public QWidget
 
 
 
-
-
 #endif
