@@ -24,6 +24,7 @@ file.
 #include <qstring.h>
 #include <qapplication.h>
 #include <qmessagebox.h>
+#include <qtranslator.h>
 #include <qmutex.h>
 #include <sys/stat.h>
 #include <sys/time.h>
@@ -45,15 +46,11 @@ using namespace std;
 #define CONFIGFILE ".extcalc/extcalc.conf"
 #define VARSFILE ".extcalc/vars.conf"
 #define LISTSFILE ".extcalc/lists.conf"
-#ifdef LANGUAGE_EN
-#define VERSIONSTRING "Version: 0.9.0\n2007-10-10\n\n"+QString(DEVVERSION)
-#endif
-#ifdef LANGUAGE_FR
-#define VERSIONSTRING "Version: 0.9.0\n2007-10-10\n\n"+QString(DEVVERSION)
-#endif
-#ifdef LANGUAGE_DE
-#define VERSIONSTRING "Version: 0.9.0\n10.10.2007\n\n"+QString(DEVVERSION)
-#endif
+#define CONSTFILE ".extcalc/constants.conf"
+#define UIFILE ".extcalc/ui.conf"
+
+#define VERSIONSTRING "Version: 0.9.1\n2008-03-08\n\n"+QString(DEVVERSION)
+
 
 #define AUTHORSTRING "Homepage:\nhttp://extcalc-linux.sourceforge.net\n\n"+QString(GLOBALH_STR1)
 
@@ -127,6 +124,8 @@ using namespace std;
 #define TABLETYPE			56
 #define STANDARDTABLE		57
 #define RESETTABLE			184
+#define GRAPHIMPORT			212
+#define GRAPHEXPORT			213
 
 #define EXPORTSCRIPT		171
 #define IMPORTSCRIPT		172
@@ -321,11 +320,17 @@ using namespace std;
 #define MPEMPTY				212
 #define MPINVALIDMACRO		213
 
+#define LANG_EN				214
+#define LANG_DE				215
+#define LANG_FR				216
+#define LPREF				217
+
 #define CATMATHSTD			0x1
 #define CATMATHCOMPLEX		0x2
 #define CATMATRIX			0x4
 #define CATSCRIPT			0x8
 #define CATMATHLOGIC		0x10
+#define CATCONSTANTS		0x20
 
 
 #define NNONE				0x1
@@ -405,6 +410,13 @@ class Math;
 
 typedef long double Variable;
 
+struct Constant
+{
+	QString *identifier;
+	QString *description;
+	QString *value;
+};
+
 
 struct Preferences
 {
@@ -454,11 +466,16 @@ struct Preferences
 	QString scriptDirName;
 	QString dataDirName;
 	QString*functions;
+	QString*functionComments;
 	bool*activeFunctions;
 	int*functionTypes;
 	bool*dynamicFunctions;
+	bool*logicFunctions;
 	QColor*functionColors;
 	QString drawText;
+	int language;
+	Constant*constList;
+	int constLen,userConstLen;
 };
 
 struct Number
@@ -520,8 +537,10 @@ char* macroPreprocessor(char*);
 char* cleanString(char*,Preferences*);
 char* removeUnicode(QString*);
 char* removeComments(char*);
+void replaceConstants(QString*,Preferences*);
 QString getConfigString(QString*,QString);
 QString cleanConfigString(QString,QString);
+QString resetConfigString(QString);
 QString getUnicode(int code);
 QString formatOutput(long double num,Preferences*pref);
 QString formatOutput(Number num,Preferences*pref,ThreadSync*varData=NULL);
@@ -532,7 +551,7 @@ long double runCalc(QString,Preferences*,Variable*);
 long double determinant(int,long double*);
 long double gauss(int sizex,int sizey,long double*matrix);
 bool invertMatrix(int size,long double*matrix);
-void printError(const char*,int,QObject*);
+void printError(const char*,int,ThreadSync*);
 void convertToFloat(Number*num);
 inline void convertToInt(Number*num);
 inline void convertToBool(Number*num);
