@@ -48,7 +48,6 @@ void GraphOutput::initializeGL()
 	if(drawState!=DRAWNONE)
 		generateTexture();
 	
-	
 	glDisable(GL_TEXTURE_2D);
 	if(drawScreenshot)
 	{
@@ -761,13 +760,16 @@ void GraphOutput::processStdFunction(QString function)
 {
 	char*func;
 	func = preprocessor(&function,&pref,false);
+	
 
 	double xStart,xStep;
+	Number result;
 	xStart=pref.xmin;
 	xStep=(pref.xmax-pref.xmin)/PRECISION2D;
 
 	QString num,num2;
-	Calculate ca1(NULL,func,&pref,vars);
+
+
 
 	struct timeval t1,t2;
 	
@@ -776,15 +778,38 @@ void GraphOutput::processStdFunction(QString function)
 	int index=objectCoordinates.GetLen()-1;
 	objectInfo[index].length=PRECISION2D;
 	objectInfo[index].function=func;
-
-	gettimeofday(&t1,NULL);
-	for(int c=0; c<PRECISION2D; c++)
+	if(objectInfo[index].logic)
 	{
-		vars[23]=xStart+(double)c*xStep;
-		objectCoordinates[index][2*c]  =vars[23];
-		objectCoordinates[index][2*c+1]=ca1.calc();
+		Script ca1(NULL,func,&pref,vars,threadData);
+		gettimeofday(&t1,NULL);
+		
+		
+		for(int c=0; c<PRECISION2D; c++)
+		{
+			threadData->vars[23][0].type=NFLOAT;
+			threadData->vars[23][0].fval=xStart+(double)c*xStep;
+			objectCoordinates[index][2*c]  =threadData->vars[23][0].fval.real();
+			result=ca1.exec();
+			convertToFloat(&result);
+			objectCoordinates[index][2*c+1]=result.fval.real();
+		}
+		gettimeofday(&t2,NULL);
 	}
-	gettimeofday(&t2,NULL);
+	else 
+	{
+		Calculate ca1(NULL,func,&pref,vars);
+		gettimeofday(&t1,NULL);
+		
+		for(int c=0; c<PRECISION2D; c++)
+		{
+			vars[23]=xStart+(double)c*xStep;
+			objectCoordinates[index][2*c]  =vars[23];
+			objectCoordinates[index][2*c+1]=ca1.calc();
+		}
+		gettimeofday(&t2,NULL);
+	}
+
+
 
 	int seconds,usecs;
 	seconds=t2.tv_sec-t1.tv_sec;
@@ -795,6 +820,7 @@ void GraphOutput::processStdFunction(QString function)
 		usecs=1000000+usecs;
 	}
 
+
 	objects.NewItem(generateGLList(index));
 }
 
@@ -825,17 +851,41 @@ void GraphOutput::processPolarFunction(QString function)
 
 	float r;
 	QString num,num2;
-	Calculate ca1(NULL,func,&pref,vars);
 	struct timeval t1,t2;
-	gettimeofday(&t1,NULL);
-	for(int c=0; c<=PRECISION2D; c++)
+	if(objectInfo[index].logic)
 	{
-		vars[23]=xStart+(double)c*xStep;
-		r=ca1.calc();
-		objectCoordinates[index][c*2]=cos(vars[23]*multiplier)*r;
-		objectCoordinates[index][c*2+1]=sin(vars[23]*multiplier)*r;
+		Number r;
+		long double x;
+		Script ca1(NULL,func,&pref,vars,threadData);
+		
+		gettimeofday(&t1,NULL);
+		
+		for(int c=0; c<=PRECISION2D; c++)
+		{
+			x=xStart+(double)c*xStep;
+			threadData->vars[23][0].type=NFLOAT;
+			threadData->vars[23][0].fval=x;
+
+			r=ca1.exec();
+			convertToFloat(&r);
+			objectCoordinates[index][2*c]  =cos(x*multiplier)*r.fval.real();
+			objectCoordinates[index][2*c+1]=sin(x*multiplier)*r.fval.real();
+			
+		}
+		gettimeofday(&t2,NULL);
+	}
+	else {
+		Calculate ca1(NULL,func,&pref,vars);
+		gettimeofday(&t1,NULL);
+		for(int c=0; c<=PRECISION2D; c++)
+		{
+			vars[23]=xStart+(double)c*xStep;
+			r=ca1.calc();
+			objectCoordinates[index][c*2]=cos(vars[23]*multiplier)*r;
+			objectCoordinates[index][c*2+1]=sin(vars[23]*multiplier)*r;
+		}
+		gettimeofday(&t2,NULL);
 	}
-	gettimeofday(&t2,NULL);
 
 	int seconds,usecs;
 	seconds=t2.tv_sec-t1.tv_sec;
@@ -869,16 +919,40 @@ void GraphOutput::processParameterFunction(QString function)
 	double xStart,xStep;
 	xStart=pref.parameterStart;
 	xStep=(pref.parameterEnd-pref.parameterStart)/(double)pref.parameterSteps;
-	
+
 	double* coordinates=new double[pref.parameterSteps*2];
 	objectCoordinates.NewItem(coordinates);
 	int index=objectCoordinates.GetLen()-1;
 	objectInfo[index].length=pref.parameterSteps;
 
 	QString num,num2;
+	struct timeval t1,t2;
+	
+	if(objectInfo[index].logic)
+	{
+		Number result;
+		Script ca1(NULL,func1,&pref,vars,threadData);
+		Script ca2(NULL,func2,&pref,vars,threadData);
+		gettimeofday(&t1,NULL);
+		
+		
+		for(int c=0; c<pref.parameterSteps; c++)
+		{
+			threadData->vars[19][0].type=NFLOAT;
+			threadData->vars[19][0].fval=xStart+(double)c*xStep;
+			result=ca1.exec();
+			convertToFloat(&result);
+			objectCoordinates[index][2*c]=result.fval.real();
+			threadData->vars[19][0].type=NFLOAT;
+			threadData->vars[19][0].fval=xStart+(double)c*xStep;
+			result=ca2.exec();
+			convertToFloat(&result);
+			objectCoordinates[index][2*c+1]=result.fval.real();
+		}
+	}
+	else {
 	Calculate ca1(NULL,func1,&pref,vars);
 	Calculate ca2(NULL,func2,&pref,vars);
-	struct timeval t1,t2;
 
 	gettimeofday(&t1,NULL);
 	for(int c=0; c<pref.parameterSteps; c++)
@@ -888,6 +962,7 @@ void GraphOutput::processParameterFunction(QString function)
 		objectCoordinates[index][2*c+1]=ca2.calc();
 	}
 	gettimeofday(&t2,NULL);
+	}
 
 	int seconds,usecs;
 	seconds=t2.tv_sec-t1.tv_sec;
@@ -962,76 +1037,46 @@ void GraphOutput::process3dFunction(QString function)
 	objectInfo[index].length=PRECISION3D;
 
 	QString num,num2;
-	Calculate ca1(NULL,func,&pref,vars);
 	struct timeval t1,t2;
-
-	gettimeofday(&t1,NULL);
-
-	for(int c=0; c<PRECISION3D; c++)
+	if(objectInfo[index].logic)
 	{
-		vars[25]=zStart+c*zStep;
-		for(int c1=0; c1<PRECISION3D; c1++)
+		Number result;
+		Script ca1(NULL,func,&pref,vars,threadData);
+		gettimeofday(&t1,NULL);
+		
+		for(int c=0; c<PRECISION3D; c++)
 		{
-			vars[23]=xStart+c1*xStep;
-			coordinates[PRECISION3D*c+c1]=ca1.calc();
+			threadData->vars[25][0].type=NFLOAT;
+			threadData->vars[25][0].fval=zStart+c*zStep;
+			for(int c1=0; c1<PRECISION3D; c1++)
+			{
+				threadData->vars[23][0].type=NFLOAT;
+				threadData->vars[23][0].fval=xStart+c1*xStep;
+				result=ca1.exec();
+				convertToFloat(&result);
+				coordinates[PRECISION3D*c+c1]=result.fval.real();
+			}
+		}
+	}
+	else {
+		Calculate ca1(NULL,func,&pref,vars);
+	
+		gettimeofday(&t1,NULL);
+	
+		for(int c=0; c<PRECISION3D; c++)
+		{
+			vars[25]=zStart+c*zStep;
+			for(int c1=0; c1<PRECISION3D; c1++)
+			{
+				vars[23]=xStart+c1*xStep;
+				coordinates[PRECISION3D*c+c1]=ca1.calc();
+			}
 		}
 	}
 
 	gettimeofday(&t2,NULL);
 
 
-	/*
-	// Code for 3D-graphs with polygons
-	glMatrixMode(GL_MODELVIEW);
-	float y;
-	float lastY[rasterSize+1];
-	QString num,num2;
-	Calculate ca1(NULL,func,&pref,vars);
-	struct timeval t1,t2;
-	bool end=false;
-
-	gettimeofday(&t1,NULL);
-	vars[25]=zStart;
-	vars[23]=xStart;
-	lastY[0]=ca1.calc();
-	for(int c=1; c<=rasterSize;c++)
-	{
-		vars[25]=zStart+(double)c*zStep;
-		lastY[c]=ca1.calc();
-	}
-	for(int c=1; c<=rasterSize;c++)
-	{
-		vars[23]=xStart+c*xStep;
-		glBegin(GL_TRIANGLE_STRIP);
-		for(int c1=0; c1<=rasterSize; c1++)
-		{
-			vars[25]=zStart+c1*zStep;
-			y=ca1.calc();
-			if((y > pref.ymin&&y<pref.ymax) || (lastY[c1] > pref.ymin && lastY[c1] < pref.ymax))
-			{
-				if(end)
-				{
-					glBegin(GL_TRIANGLE_STRIP);
-					end=false;
-				}
-				if(colored)
-					setGLColor(lastY[c1]);
-				glVertex3f(xStart+(c-1)*xStep,lastY[c1],vars[25]);
-				if(colored)
-					setGLColor(y);
-				glVertex3f(xStart+(c)*xStep,y,vars[25]);
-			}
-			else {
-				glEnd();
-				end=true;
-			}
-
-			lastY[c1]=y;
-		}
-		glEnd();
-	}
-	//end
-	*/	
 	glEndList();
 
 	int seconds,usecs;
@@ -1052,7 +1097,7 @@ void GraphOutput::process3dFunction(QString function)
 }
 
 
-void GraphOutput::processInequaityFunction(QString function1,QString function2,int type)
+void GraphOutput::processInequalityFunction(QString function1,QString function2,int type)
 {
 	//types:
 	//GRAPHIEG: both greather
@@ -1087,19 +1132,42 @@ void GraphOutput::processInequaityFunction(QString function1,QString function2,i
 	objectInfo[index].length=PRECISION2D+1;
 	objectInfo[index].function=func;
 	objectInfo[index].function2=func2;
-	
+
 	float x,y1,y2;
 	QString num,num2;
-	Calculate ca1(NULL,func,&pref,vars);
-	Calculate ca2(NULL,func2,&pref,vars);
-	
+	Calculate *ca1=NULL,*ca2=NULL;
+	Script*s1=NULL,*s2=NULL;
+	Number result;
+	if(objectInfo[index].logic)
+	{
+		
+		s1=new Script(NULL,func,&pref,vars,threadData);	
+		s2=new Script(NULL,func2,&pref,vars,threadData);	
+	}
+	else {
+		ca1=new Calculate(NULL,func,&pref,vars);
+		ca2=new Calculate(NULL,func2,&pref,vars);
+	}
 	for(int c=0; c<=PRECISION2D; c++)
 	{
 		x=xStart+(double)c*xStep;
-		vars[23]=x;
-		y1=ca1.calc();
-		y2=ca2.calc();
-		
+
+		if(objectInfo[index].logic)
+		{
+			threadData->vars[23][0].type=NFLOAT;
+			threadData->vars[23][0].fval=x;
+			result=s1->exec();
+			y1=result.fval.real();
+			threadData->vars[23][0].type=NFLOAT;
+			threadData->vars[23][0].fval=x;
+			result=s2->exec();
+			y2=result.fval.real();
+		}
+		else {
+			vars[23]=x;
+			y1=ca1->calc();
+			y2=ca2->calc();
+		}
 		if(type==GRAPHIEG)
 		{
 			objectCoordinates[index][2*c]=x;
@@ -1139,6 +1207,15 @@ void GraphOutput::processInequaityFunction(QString function1,QString function2,i
 //	MessageBox(  "Sekunden:      "+QString::number(seconds)+
 //			   "\nMicrosekunden: "+QString::number(usecs));
 	objects.NewItem(generateGLList(index));
+	if(objectInfo[index].logic)
+	{
+		delete s1;
+		delete s2;	
+	}
+	else {
+		delete ca1;
+		delete ca2;
+	}
 }
 
 void GraphOutput::processComplexFunction(QString function,bool draw3D=false)
@@ -1167,8 +1244,6 @@ void GraphOutput::processComplexFunction(QString function,bool draw3D=false)
 	pref.complex=oldcpref;
 	struct timeval t1,t2;
 	Number result;
-	
-	
 
 	objectCoordinates.NewItem(coordinates);
 	int index=objectCoordinates.GetLen()-1;
@@ -1181,7 +1256,6 @@ void GraphOutput::processComplexFunction(QString function,bool draw3D=false)
 	{
 		for(int c=0; c<=pref.nyquistSteps; c++)
 		{
-			
 			threadData->vars[25][0].fval=Complex(zStart+(double)c*zStep,0.0);
 			result=ca1.exec();
 			objectCoordinates[index][3*c+2]  =threadData->vars[25][0].fval.real();
@@ -1200,7 +1274,7 @@ void GraphOutput::processComplexFunction(QString function,bool draw3D=false)
 			objectCoordinates[index][2*c+1]=result.fval.imag();
 		}
 	}
-	
+
 	gettimeofday(&t2,NULL);
 
 	int seconds,usecs;
@@ -1485,34 +1559,66 @@ bool GraphOutput::updateFunctions(double oldXMin,double oldXMax)
 
 	for(int c=0; c<objectInfo.GetLen(); c++)
 	{
-		vars[0]=objectInfo[c].dynamicParameter;
+		if(objectInfo[c].logic)
+		{
+			threadData->vars[0][0].type=NFLOAT;
+			threadData->vars[0][0].fval=Complex(dynamicStart+c*((dynamicEnd-dynamicStart)/(dynamicSteps)),0.0);
+		}
+		else vars[0]=objectInfo[c].dynamicParameter;
+		
 		if(!ret);
 		else if(objectInfo[c].type == GRAPHSTD)
 		{
 //			perror("xStart: "+QString::number(xStart)+" xStep: "+QString::number(xStep)+" shiftRight: "+QString::number(shiftRight)+" function: "+QString(objectInfo[c].function)); 
 
-			Calculate ca(NULL,objectInfo[c].function,&pref,vars);
+			
 
 			if(shiftRight)
 				for(int c1=steps*2; c1<PRECISION2D*2; c1++)
 					objectCoordinates[c][c1-steps*2]=objectCoordinates[c][c1];
 			else for(int c1=(PRECISION2D)*2+1; c1>=0; c1--)
 						objectCoordinates[c][c1]=objectCoordinates[c][c1-steps*2];
-
-			for(int c1=0; c1<steps; c1++)
+			if(objectInfo[c].logic)
 			{
-				vars[23]=xStart+c1*xStep;
-				objectCoordinates[c][(startStep+c1)*2]=vars[23];
-				objectCoordinates[c][(startStep+c1)*2+1]=ca.calc();
+				Number result;
+				Script ca(NULL,objectInfo[c].function,&pref,vars,threadData);
+				for(int c1=0; c1<steps; c1++)
+				{
+					threadData->vars[23][0].type=NFLOAT;
+					threadData->vars[23][0].fval=xStart+c1*xStep;
+					objectCoordinates[c][(startStep+c1)*2]  =threadData->vars[23][0].fval.real();
+					result=ca.exec();
+					convertToFloat(&result);
+					objectCoordinates[c][(startStep+c1)*2+1]=result.fval.real();
+				}
+			}
+			else {
+				Calculate ca(NULL,objectInfo[c].function,&pref,vars);
+				for(int c1=0; c1<steps; c1++)
+				{
+					vars[23]=xStart+c1*xStep;
+					objectCoordinates[c][(startStep+c1)*2]=vars[23];
+					objectCoordinates[c][(startStep+c1)*2+1]=ca.calc();
+				}
 			}
 		}
 		else if(objectInfo[c].type == GRAPHIEG || objectInfo[c].type == GRAPHIEL || objectInfo[c].type == GRAPHIEGE || objectInfo[c].type == GRAPHIELE)
 		{
 //			perror("xStart: "+QString::number(xStart)+" xStep: "+QString::number(xStep)+" shiftRight: "+QString::number(shiftRight)+" function: "+QString(objectInfo[c].function)); 
 
-					
-			Calculate ca(NULL,objectInfo[c].function,&pref,vars);
-			Calculate ca2(NULL,objectInfo[c].function2,&pref,vars);
+			Calculate *ca1=NULL,*ca2=NULL;
+			Script*s1=NULL,*s2=NULL;
+			Number result;
+			if(objectInfo[c].logic)
+			{
+				
+				s1=new Script(NULL,objectInfo[c].function,&pref,vars,threadData);	
+				s2=new Script(NULL,objectInfo[c].function2,&pref,vars,threadData);	
+			}
+			else {
+				ca1=new Calculate(NULL,objectInfo[c].function,&pref,vars);
+				ca2=new Calculate(NULL,objectInfo[c].function2,&pref,vars);
+			}
 			
 			if(objectInfo[c].type == GRAPHIEG || objectInfo[c].type == GRAPHIEL)
 			{
@@ -1537,9 +1643,22 @@ bool GraphOutput::updateFunctions(double oldXMin,double oldXMax)
 			for(int c1=0; c1<steps; c1++)
 			{
 				x=xStart+(double)c1*xStep;
-				vars[23]=x;
-				y1=ca.calc();
-				y2=ca2.calc();
+				if(objectInfo[c].logic)
+				{
+					threadData->vars[23][0].type=NFLOAT;
+					threadData->vars[23][0].fval=x;
+					result=s1->exec();
+					y1=result.fval.real();
+					threadData->vars[23][0].type=NFLOAT;
+					threadData->vars[23][0].fval=x;
+					result=s2->exec();
+					y2=result.fval.real();
+				}
+				else {
+					vars[23]=x;
+					y1=ca1->calc();
+					y2=ca2->calc();
+				}
 		
 				if(objectInfo[c].type==GRAPHIEG)
 				{
@@ -1566,6 +1685,15 @@ bool GraphOutput::updateFunctions(double oldXMin,double oldXMax)
 					else objectCoordinates[c][3*(startStep+c1)+1]=objectCoordinates[c][3*(startStep+c1)+2]=0.0;
 				}
 			}
+			if(objectInfo[c].logic)
+			{
+				delete s1;
+				delete s2;	
+			}
+			else {
+				delete ca1;
+				delete ca2;
+			}
 			if(shiftRight)
 				steps--;
 		}
@@ -1599,6 +1727,7 @@ void GraphOutput::processFunction(int index)
 		info.type=pref.functionTypes[index];
 		info.color=pref.functionColors[index];
 		info.dynamic=false;
+		info.logic=pref.logicFunctions[index];
 		objectInfo.NewItem(info);
 		
 		drawRules.NewItem(new int[2]);
@@ -1631,7 +1760,7 @@ void GraphOutput::processFunction(int index)
 					{
 						objectInfo[objectInfo.GetLen()-1].type=GRAPHIEG;
 						objectInfo[objectInfo.GetLen()-1].color=QColor(123,121,255);
-						processInequaityFunction(pref.functions[ineq1],pref.functions[ineq2],GRAPHIEG);
+						processInequalityFunction(pref.functions[ineq1],pref.functions[ineq2],GRAPHIEG);
 						
 					}
 					else if((pref.functionTypes[ineq1]==GRAPHIEL || pref.functionTypes[ineq1]==GRAPHIELE) &&
@@ -1639,26 +1768,27 @@ void GraphOutput::processFunction(int index)
 					{
 						objectInfo[objectInfo.GetLen()-1].type=GRAPHIEL;
 						objectInfo[objectInfo.GetLen()-1].color=QColor(123,121,255);
-						processInequaityFunction(pref.functions[ineq1],pref.functions[ineq2],GRAPHIEL);
+						processInequalityFunction(pref.functions[ineq1],pref.functions[ineq2],GRAPHIEL);
 					}
 					else if((pref.functionTypes[ineq1]==GRAPHIEL || pref.functionTypes[ineq1]==GRAPHIELE) &&
 						(pref.functionTypes[ineq2]==GRAPHIEG || pref.functionTypes[ineq2]==GRAPHIEGE))
 					{
 						objectInfo[objectInfo.GetLen()-1].type=GRAPHIEGE;
 						objectInfo[objectInfo.GetLen()-1].color=QColor(123,121,255);
-						processInequaityFunction(pref.functions[ineq2],pref.functions[ineq1],GRAPHIEGE);
+						processInequalityFunction(pref.functions[ineq2],pref.functions[ineq1],GRAPHIEGE);
 					}
 					else
 					{
 						objectInfo[objectInfo.GetLen()-1].type=GRAPHIEGE;
 						objectInfo[objectInfo.GetLen()-1].color=QColor(123,121,255);
-						processInequaityFunction(pref.functions[ineq1],pref.functions[ineq2],GRAPHIEGE);
+						processInequalityFunction(pref.functions[ineq1],pref.functions[ineq2],GRAPHIEGE);
 					}
 					drawRules[ruleIndex][1]=objects.GetLen()-1;
 					
 					info.type=GRAPHSTD;
 					info.color=pref.functionColors[ineq1];
 					info.dynamic=false;
+					info.logic=pref.logicFunctions[ineq1]|pref.logicFunctions[ineq2];
 					objectInfo.NewItem(info);
 					drawRules.NewItem(new int[2]);
 					drawRules[ruleIndex+1][0]=1;
@@ -1667,6 +1797,7 @@ void GraphOutput::processFunction(int index)
 					
 					info.type=GRAPHSTD;
 					info.color=pref.functionColors[ineq2];
+					info.logic=pref.logicFunctions[ineq2];
 					info.dynamic=false;
 					objectInfo.NewItem(info);
 					drawRules.NewItem(new int[2]);
@@ -1679,17 +1810,18 @@ void GraphOutput::processFunction(int index)
 					if(pref.functionTypes[index]==GRAPHIEG ||pref.functionTypes[index]==GRAPHIEGE)
 					{
 						objectInfo[objectInfo.GetLen()-1].type=GRAPHIEG;
-						processInequaityFunction(pref.functions[index],pref.functions[index],GRAPHIEG);
+						processInequalityFunction(pref.functions[index],pref.functions[index],GRAPHIEG);
 					}
 					else {
 						objectInfo[objectInfo.GetLen()-1].type=GRAPHIEL;
-						processInequaityFunction(pref.functions[index],pref.functions[index],GRAPHIEL);
+						processInequalityFunction(pref.functions[index],pref.functions[index],GRAPHIEL);
 					}
 					drawRules[ruleIndex][1]=objects.GetLen()-1;
 
 					info.type=GRAPHSTD;
 					info.color=QColor(0,0,0);
 					info.dynamic=false;
+					info.logic=pref.logicFunctions[index];
 					objectInfo.NewItem(info);
 					drawRules.NewItem(new int[2]);
 					drawRules[ruleIndex+1][0]=1;
@@ -1718,6 +1850,7 @@ void GraphOutput::processFunction(int index)
 		ObjectInfo info;
 		info.type=pref.functionTypes[index];
 		info.color=pref.functionColors[index];
+		info.logic=pref.logicFunctions[index];
 		info.dynamic=true;
 
 		
@@ -1730,8 +1863,13 @@ void GraphOutput::processFunction(int index)
 			case GRAPHPOLAR:
 				for(int c=0; c<=dynamicSteps; c++)
 				{
-					vars[0]=dynamicStart+c*((dynamicEnd-dynamicStart)/(dynamicSteps));
-					info.dynamicParameter=vars[0];
+					if(pref.logicFunctions[index])
+					{
+						threadData->vars[0][0].type=NFLOAT;
+						threadData->vars[0][0].fval=Complex(dynamicStart+c*((dynamicEnd-dynamicStart)/(dynamicSteps)),0.0);
+					}
+					else vars[0]=dynamicStart+c*((dynamicEnd-dynamicStart)/(dynamicSteps));
+					info.dynamicParameter=dynamicStart+c*((dynamicEnd-dynamicStart)/(dynamicSteps));
 					objectInfo.NewItem(info);
 					processPolarFunction(pref.functions[index]);
 					drawRules[ruleIndex][c+1]=objects.GetLen()-1;
@@ -1740,8 +1878,13 @@ void GraphOutput::processFunction(int index)
 			case GRAPHPARAMETER:
 				for(int c=0; c<=dynamicSteps; c++)
 				{
-					vars[0]=dynamicStart+c*((dynamicEnd-dynamicStart)/(dynamicSteps));
-					info.dynamicParameter=vars[0];
+					if(pref.logicFunctions[index])
+					{
+						threadData->vars[0][0].type=NFLOAT;
+						threadData->vars[0][0].fval=Complex(dynamicStart+c*((dynamicEnd-dynamicStart)/(dynamicSteps)),0.0);
+					}
+					else vars[0]=dynamicStart+c*((dynamicEnd-dynamicStart)/(dynamicSteps));
+					info.dynamicParameter=dynamicStart+c*((dynamicEnd-dynamicStart)/(dynamicSteps));
 					objectInfo.NewItem(info);
 					processParameterFunction(pref.functions[index]);
 					drawRules[ruleIndex][c+1]=objects.GetLen()-1;
@@ -1772,8 +1915,13 @@ void GraphOutput::processFunction(int index)
 			case GRAPH3D:
 				for(int c=0; c<=dynamicSteps; c++)
 				{
-					vars[0]=dynamicStart+c*((dynamicEnd-dynamicStart)/(dynamicSteps));
-					info.dynamicParameter=vars[0];
+					if(pref.logicFunctions[index])
+					{
+						threadData->vars[0][0].type=NFLOAT;
+						threadData->vars[0][0].fval=Complex(dynamicStart+c*((dynamicEnd-dynamicStart)/(dynamicSteps)),0.0);
+					}
+					else vars[0]=dynamicStart+c*((dynamicEnd-dynamicStart)/(dynamicSteps));
+					info.dynamicParameter=dynamicStart+c*((dynamicEnd-dynamicStart)/(dynamicSteps));
 					objectInfo.NewItem(info);
 					process3dFunction(pref.functions[index]);
 					drawRules[ruleIndex][c+1]=objects.GetLen()-1;
@@ -1786,10 +1934,16 @@ void GraphOutput::processFunction(int index)
 				
 				for(int c=0; c<=dynamicSteps; c++)
 				{
-					vars[0]=dynamicStart+c*((dynamicEnd-dynamicStart)/(dynamicSteps));
-					info.dynamicParameter=vars[0];
+					if(pref.logicFunctions[index])
+					{
+						threadData->vars[0][0].type=NFLOAT;
+						threadData->vars[0][0].fval=Complex(dynamicStart+c*((dynamicEnd-dynamicStart)/(dynamicSteps)),0.0);
+					}
+					else vars[0]=dynamicStart+c*((dynamicEnd-dynamicStart)/(dynamicSteps));
+					info.dynamicParameter=dynamicStart+c*((dynamicEnd-dynamicStart)/(dynamicSteps));
 					info.type=pref.functionTypes[index];
 					info.color=pref.functionColors[index];
+					info.logic=pref.logicFunctions[index];
 					info.dynamic=true;
 					objectInfo.NewItem(info);
 
@@ -1800,7 +1954,7 @@ void GraphOutput::processFunction(int index)
 						{
 							objectInfo[objectInfo.GetLen()-1].type=GRAPHIEG;
 							objectInfo[objectInfo.GetLen()-1].color=QColor(123,121,255);
-							processInequaityFunction(pref.functions[ineq1],pref.functions[ineq2],GRAPHIEG);
+							processInequalityFunction(pref.functions[ineq1],pref.functions[ineq2],GRAPHIEG);
 					
 						}
 						else if((pref.functionTypes[ineq1]==GRAPHIEL || pref.functionTypes[ineq1]==GRAPHIELE) &&
@@ -1808,26 +1962,27 @@ void GraphOutput::processFunction(int index)
 						{
 							objectInfo[objectInfo.GetLen()-1].type=GRAPHIEL;
 							objectInfo[objectInfo.GetLen()-1].color=QColor(123,121,255);
-							processInequaityFunction(pref.functions[ineq1],pref.functions[ineq2],GRAPHIEL);
+							processInequalityFunction(pref.functions[ineq1],pref.functions[ineq2],GRAPHIEL);
 						}
 						else if((pref.functionTypes[ineq1]==GRAPHIEL || pref.functionTypes[ineq1]==GRAPHIELE) &&
 														(pref.functionTypes[ineq2]==GRAPHIEG || pref.functionTypes[ineq2]==GRAPHIEGE))
 						{
 							objectInfo[objectInfo.GetLen()-1].type=GRAPHIEGE;
 							objectInfo[objectInfo.GetLen()-1].color=QColor(123,121,255);
-							processInequaityFunction(pref.functions[ineq2],pref.functions[ineq1],GRAPHIEGE);
+							processInequalityFunction(pref.functions[ineq2],pref.functions[ineq1],GRAPHIEGE);
 						}
 						else
 						{
 							objectInfo[objectInfo.GetLen()-1].type=GRAPHIEGE;
 							objectInfo[objectInfo.GetLen()-1].color=QColor(123,121,255);
-							processInequaityFunction(pref.functions[ineq1],pref.functions[ineq2],GRAPHIEGE);
+							processInequalityFunction(pref.functions[ineq1],pref.functions[ineq2],GRAPHIEGE);
 						}
 						drawRules[ruleIndex][c+1]=objects.GetLen()-1;
 				
 						info.type=GRAPHSTD;
 						info.color=pref.functionColors[ineq1];
-						info.dynamic=false;
+						info.dynamic=true;
+						info.logic=pref.logicFunctions[ineq1]|pref.logicFunctions[ineq2];
 						objectInfo.NewItem(info);
 						if(c==0)
 						{
@@ -1839,7 +1994,8 @@ void GraphOutput::processFunction(int index)
 				
 						info.type=GRAPHSTD;
 						info.color=pref.functionColors[ineq2];
-						info.dynamic=false;
+						info.dynamic=true;
+						info.logic=pref.logicFunctions[ineq2];
 						objectInfo.NewItem(info);
 						if(c==0)
 						{
@@ -1854,17 +2010,18 @@ void GraphOutput::processFunction(int index)
 						if(pref.functionTypes[index]==GRAPHIEG ||pref.functionTypes[index]==GRAPHIEGE)
 						{
 							objectInfo[objectInfo.GetLen()-1].type=GRAPHIEG;
-							processInequaityFunction(pref.functions[index],pref.functions[index],GRAPHIEG);
+							processInequalityFunction(pref.functions[index],pref.functions[index],GRAPHIEG);
 						}
 						else {
 							objectInfo[objectInfo.GetLen()-1].type=GRAPHIEL;
-							processInequaityFunction(pref.functions[index],pref.functions[index],GRAPHIEL);
+							processInequalityFunction(pref.functions[index],pref.functions[index],GRAPHIEL);
 						}
 						drawRules[ruleIndex][c+1]=objects.GetLen()-1;
 
 						info.type=GRAPHSTD;
 						info.color=QColor(0,0,0);
-						info.dynamic=false;
+						info.dynamic=true;
+						info.logic=pref.logicFunctions[index];
 						objectInfo.NewItem(info);
 						if(c==0)
 						{
@@ -1879,8 +2036,13 @@ void GraphOutput::processFunction(int index)
 			default:
 				for(int c=0; c<=dynamicSteps; c++)
 				{
-					vars[0]=dynamicStart+c*((dynamicEnd-dynamicStart)/(dynamicSteps));
-					info.dynamicParameter=vars[0];
+					if(pref.logicFunctions[index])
+					{
+						threadData->vars[0][0].type=NFLOAT;
+						threadData->vars[0][0].fval=Complex(dynamicStart+c*((dynamicEnd-dynamicStart)/(dynamicSteps)),0.0);
+					}
+					else vars[0]=dynamicStart+c*((dynamicEnd-dynamicStart)/(dynamicSteps));
+					info.dynamicParameter=dynamicStart+c*((dynamicEnd-dynamicStart)/(dynamicSteps));
 					objectInfo.NewItem(info);
 					processStdFunction(pref.functions[index]);
 					drawRules[ruleIndex][c+1]=objects.GetLen()-1;
@@ -1891,6 +2053,105 @@ void GraphOutput::processFunction(int index)
 	}
 }
 
+inline void GraphOutput::drawTriangle(float x[3],float y[3],float z[3],bool colored)
+{
+	int c;
+	if(y[0]>pref.ymin && y[0]<pref.ymax &&		//all vertex inside c s
+		   y[1]>pref.ymin && y[1]<pref.ymax &&
+		   y[2]>pref.ymin && y[2]<pref.ymax)
+	{
+		for(c=0; c<3; c++)
+		{
+			if(colored)
+				setGLColor(y[c]);
+			glVertex3f(x[c],y[c],z[c]);
+		}
+	}
+	if((y[0]>pref.ymax || y[1]>pref.ymax || y[2]>pref.ymax) &&	//one vertex over maximum, one below minimum
+		   (y[0]<pref.ymin || y[1]<pref.ymin || y[2]<pref.ymin))
+	{
+		return;
+	}
+	else if((y[0]>pref.ymin && y[0]<pref.ymax ||		//at least one vertex inside, at least one outside c s
+				y[1]>pref.ymin && y[1]<pref.ymax ||
+				y[2]>pref.ymin && y[2]<pref.ymax) && 
+				(y[0]<pref.ymin || y[0]>pref.ymax ||
+				y[1]<pref.ymin || y[1]>pref.ymax ||
+				y[2]<pref.ymin || y[2]>pref.ymax))
+	{
+		int vout[2],vin[2],outcount=0;
+		for(int c=0; c<3; c++)
+		{
+			if(y[c]>pref.ymax || y[c]<pref.ymin)
+			{
+				vout[outcount]=c;
+				outcount++;
+			}
+			else vin[c-outcount]=c;
+		}
+		
+		double tmpx[2],tmpy[2],tmpz[2],fakt,ygr;
+		if(y[vout[0]]>pref.ymax)
+			ygr=pref.ymax;
+		else ygr=pref.ymin;
+		
+		if(outcount==1)
+		{
+
+			
+			for(int c=0; c<2; c++)
+			{
+				fakt=(ygr-y[vin[c]])/(y[vout[0]]-y[vin[c]]);
+				
+				tmpx[c]=x[vin[c]]+(x[vout[0]]-x[vin[c]])*fakt;
+				tmpz[c]=z[vin[c]]+(z[vout[0]]-z[vin[c]])*fakt;
+				tmpy[c]=ygr;
+			}
+			if(colored)
+				setGLColor(y[vin[0]]);
+			glVertex3f(x[vin[0]],y[vin[0]],z[vin[0]]);
+			if(colored)
+				setGLColor(y[vin[1]]);
+			glVertex3f(x[vin[1]],y[vin[1]],z[vin[1]]);
+			if(colored)
+				setGLColor(tmpy[0]);
+			glVertex3f(tmpx[0],tmpy[0],tmpz[0]);
+
+			if(colored)
+				setGLColor(tmpy[0]);
+			glVertex3f(tmpx[0],tmpy[0],tmpz[0]);
+			if(colored)
+				setGLColor(tmpy[1]);
+			glVertex3f(tmpx[1],tmpy[1],tmpz[1]);
+			if(colored)
+				setGLColor(y[vin[1]]);
+			glVertex3f(x[vin[1]],y[vin[1]],z[vin[1]]);
+		}
+		else
+		{
+			for(int c=0; c<2; c++)
+			{
+				fakt=(ygr-y[vin[0]])/(y[vout[c]]-y[vin[0]]);
+				
+				tmpx[c]=x[vin[0]]+(x[vout[c]]-x[vin[0]])*fakt;
+				tmpz[c]=z[vin[0]]+(z[vout[c]]-z[vin[0]])*fakt;
+				tmpy[c]=ygr;
+			}
+
+			if(colored)
+				setGLColor(tmpy[0]);
+			glVertex3f(tmpx[0],tmpy[0],tmpz[0]);
+			if(colored)
+				setGLColor(tmpy[1]);
+			glVertex3f(tmpx[1],tmpy[1],tmpz[1]);
+			if(colored)
+				setGLColor(y[vin[0]]);
+			glVertex3f(x[vin[0]],y[vin[0]],z[vin[0]]);
+		}
+	}
+}
+
+
 GLuint GraphOutput::generateGLList(int index)
 {
 	GLuint list;
@@ -1931,97 +2192,143 @@ GLuint GraphOutput::generateGLList(int index)
 		if(objectCoordinates[index]!=NULL)
 		{
 			glMatrixMode(GL_MODELVIEW);
-			for(int c=0; c<PRECISION3D; c++)
+			if(!pref.show3dGrid)
 			{
-				z=zStart+c*zStep;
-				glBegin(GL_LINES);
-				lastY=objectCoordinates[index][c*PRECISION3D];
-				for(int c1=1; c1<PRECISION3D; c1++)
+				double z2,x2;
+				float xc[3],yc[3],zc[3];
+				glBegin(GL_TRIANGLES);
+				for(int c=1; c<PRECISION3D; c++)			//Z
 				{
-					x=xStart+c1*xStep;
-					y=objectCoordinates[index][c*PRECISION3D+c1];
-					
-					if((y > pref.ymin&&y<pref.ymax) || (lastY > pref.ymin && lastY < pref.ymax))
+					z=zStart+c*zStep;
+					z2=zStart+(c-1)*zStep;
+					for(int c1=1; c1<PRECISION3D; c1++)		//X
 					{
-						double newXStep=xStep,oldXStep=0.0,newY=y,oldY=lastY;
-						if(y<pref.ymin && lastY>pref.ymin)
-						{
-							newY=pref.ymin;
-							newXStep=xStep*(oldY-pref.ymin)/(oldY-y);
-						}
-						else if(y>pref.ymax && lastY<pref.ymax)
-						{
-							newY=pref.ymax;
-							newXStep=xStep*(pref.ymax-oldY)/(y-oldY);
-						}
-						else if(y>pref.ymin && lastY<pref.ymin)
-						{
-							oldY=pref.ymin;
-							oldXStep=xStep*(pref.ymin-oldY)/(y-oldY);
-						}
-						else if(y<pref.ymax && lastY>pref.ymax)
-						{
-							oldY=pref.ymax;
-							oldXStep=xStep*(oldY-pref.ymax)/(oldY-y);
-						}
-						oldXStep+=xStep*(c1-1)+xStart;
-						newXStep+=xStep*(c1-1)+xStart;
-						if(colored)
-							setGLColor(oldY);
-						glVertex3f(oldXStep,oldY,z);
-						if(colored)
-							setGLColor(newY);
-						glVertex3f(newXStep,newY,z);
+						x=xStart+c1*xStep;
+						x2=xStart+(c1-1)*xStep;
+						xc[0]=x2;
+						xc[1]=x2;
+						xc[2]=x;
+						
+						yc[0]=objectCoordinates[index][(c-1)*PRECISION3D+c1-1];
+						yc[1]=objectCoordinates[index][c*PRECISION3D+c1-1];
+						yc[2]=objectCoordinates[index][(c-1)*PRECISION3D+c1];
+						
+						zc[0]=z2;
+						zc[1]=z;
+						zc[2]=z2;
+						drawTriangle(xc,yc,zc,colored);
+
+						xc[0]=x2;
+						xc[1]=x;
+						xc[2]=x;
+						
+						yc[0]=objectCoordinates[index][c*PRECISION3D+c1-1];
+						yc[1]=objectCoordinates[index][c*PRECISION3D+c1];
+						yc[2]=objectCoordinates[index][(c-1)*PRECISION3D+c1];
+						
+						zc[0]=z;
+						zc[1]=z;
+						zc[2]=z2;
+						drawTriangle(xc,yc,zc,colored);
 					}
-					lastY=y;
+					
 				}
 				glEnd();
 			}
-			for(int c=0; c<PRECISION3D; c++)
+			else 
 			{
-				x=xStart+c*xStep;
-				glBegin(GL_LINES);
-				lastY=objectCoordinates[index][c];
-				for(int c1=1; c1<PRECISION3D; c1++)
+				for(int c=0; c<PRECISION3D; c++)
 				{
-					z=zStart+c1*zStep;
-					y=objectCoordinates[index][c1*PRECISION3D+c];
-					
-					if((y > pref.ymin&&y<pref.ymax) || (lastY > pref.ymin && lastY < pref.ymax))
+					z=zStart+c*zStep;
+					glBegin(GL_LINES);
+					lastY=objectCoordinates[index][c*PRECISION3D];
+					for(int c1=1; c1<PRECISION3D; c1++)
 					{
-						double newZStep=zStep,oldZStep=0.0,newY=y,oldY=lastY;
-						if(y<pref.ymin && lastY>pref.ymin)
+						x=xStart+c1*xStep;
+						y=objectCoordinates[index][c*PRECISION3D+c1];
+						
+						if((y > pref.ymin&&y<pref.ymax) || (lastY > pref.ymin && lastY < pref.ymax))
 						{
-							newY=pref.ymin;
-							newZStep=zStep*(oldY-pref.ymin)/(oldY-y);
+							double newXStep=xStep,oldXStep=0.0,newY=y,oldY=lastY;
+							if(y<pref.ymin && lastY>pref.ymin)
+							{
+								newY=pref.ymin;
+								newXStep=xStep*(oldY-pref.ymin)/(oldY-y);
+							}
+							else if(y>pref.ymax && lastY<pref.ymax)
+							{
+								newY=pref.ymax;
+								newXStep=xStep*(pref.ymax-oldY)/(y-oldY);
+							}
+							else if(y>pref.ymin && lastY<pref.ymin)
+							{
+								oldY=pref.ymin;
+								oldXStep=xStep*(pref.ymin-oldY)/(y-oldY);
+							}
+							else if(y<pref.ymax && lastY>pref.ymax)
+							{
+								oldY=pref.ymax;
+								oldXStep=xStep*(oldY-pref.ymax)/(oldY-y);
+							}
+							oldXStep+=xStep*(c1-1)+xStart;
+							newXStep+=xStep*(c1-1)+xStart;
+							if(colored)
+								setGLColor(oldY);
+							glVertex3f(oldXStep,oldY,z);
+							if(colored)
+								setGLColor(newY);
+							glVertex3f(newXStep,newY,z);
 						}
-						else if(y>pref.ymax && lastY<pref.ymax)
-						{
-							newY=pref.ymax;
-							newZStep=zStep*(pref.ymax-oldY)/(y-oldY);
-						}
-						else if(y>pref.ymin && lastY<pref.ymin)
-						{
-							oldY=pref.ymin;
-							oldZStep=zStep*(pref.ymin-oldY)/(y-oldY);
-						}
-						else if(y<pref.ymax && lastY>pref.ymax)
-						{
-							oldY=pref.ymax;
-							oldZStep=zStep*(oldY-pref.ymax)/(oldY-y);
-						}
-						oldZStep+=zStep*(c1-1)+zStart;
-						newZStep+=zStep*(c1-1)+zStart;
-						if(colored)
-							setGLColor(oldY);
-						glVertex3f(x,oldY,oldZStep);
-						if(colored)
-							setGLColor(newY);
-						glVertex3f(x,newY,newZStep);
+						lastY=y;
 					}
-					lastY=y;
+					glEnd();
+				}
+				for(int c=0; c<PRECISION3D; c++)
+				{
+					x=xStart+c*xStep;
+					glBegin(GL_LINES);
+					lastY=objectCoordinates[index][c];
+					for(int c1=1; c1<PRECISION3D; c1++)
+					{
+						z=zStart+c1*zStep;
+						y=objectCoordinates[index][c1*PRECISION3D+c];
+						
+						if((y > pref.ymin&&y<pref.ymax) || (lastY > pref.ymin && lastY < pref.ymax))
+						{
+							double newZStep=zStep,oldZStep=0.0,newY=y,oldY=lastY;
+							if(y<pref.ymin && lastY>pref.ymin)
+							{
+								newY=pref.ymin;
+								newZStep=zStep*(oldY-pref.ymin)/(oldY-y);
+							}
+							else if(y>pref.ymax && lastY<pref.ymax)
+							{
+								newY=pref.ymax;
+								newZStep=zStep*(pref.ymax-oldY)/(y-oldY);
+							}
+							else if(y>pref.ymin && lastY<pref.ymin)
+							{
+								oldY=pref.ymin;
+								oldZStep=zStep*(pref.ymin-oldY)/(y-oldY);
+							}
+							else if(y<pref.ymax && lastY>pref.ymax)
+							{
+								oldY=pref.ymax;
+								oldZStep=zStep*(oldY-pref.ymax)/(oldY-y);
+							}
+							oldZStep+=zStep*(c1-1)+zStart;
+							newZStep+=zStep*(c1-1)+zStart;
+							if(colored)
+								setGLColor(oldY);
+							glVertex3f(x,oldY,oldZStep);
+							if(colored)
+								setGLColor(newY);
+							glVertex3f(x,newY,newZStep);
+						}
+						lastY=y;
+					}
+					glEnd();
 				}
-				glEnd();
 			}
 		}
 		glEndList();
@@ -2387,7 +2694,7 @@ void GraphOutput::draw3dXLine(double y,double z)
 
 void GraphOutput::draw3dYLine(double x,double z)
 {
-	if(x<pref.ymin || x>pref.ymax || z<pref.zmin || z>pref.zmax)
+	if(x<pref.xmin || x>pref.xmax || z<pref.zmin || z>pref.zmax)
 		return;
 	GLuint list;
 	list = glGenLists( 1 );
@@ -2404,7 +2711,7 @@ void GraphOutput::draw3dYLine(double x,double z)
 }
 void GraphOutput::draw3dZLine(double x,double y)
 {
-	if(x<pref.ymin || x>pref.ymax || y<pref.zmin || y>pref.zmax)
+	if(x<pref.xmin || x>pref.xmax || y<pref.ymin || y>pref.ymax)
 		return;
 	GLuint list;
 	list = glGenLists( 1 );
@@ -2449,7 +2756,6 @@ void GraphOutput::drawPoints(long double *coordinates,int num,bool con)
 	if(!drawScreenshot)
 		repaint();
 	hasStatisticsObjects=true;
-	
 }
 
 
