@@ -65,6 +65,7 @@ struct ObjectInfo
 {
 	int type;
 	bool dynamic;
+	bool logic;
 	double dynamicParameter;
 	int length;
 	char* function;
@@ -164,7 +165,7 @@ public:
 	void processPolarFunction(QString);
 	void processParameterFunction(QString);
 	void process3dFunction(QString);
-	void processInequaityFunction(QString,QString,int);
+	void processInequalityFunction(QString,QString,int);
 	void processComplexFunction(QString,bool);
 	void processFunction(int);
 	bool updateFunctions(double,double);
@@ -177,6 +178,9 @@ public:
 	void setPref(Preferences newPref);
 	void clearGL();
 	void setGLColor(float y);
+	
+private:
+	inline void drawTriangle(float[3],float[3],float[3],bool colored);
 
 public slots:
 	void drawHorizontalLine(double y);
