@@ -1,3 +1,18 @@
+/*/////////////////////////////////////////Extcalc////////////////////////////////////////////
+/////////////////////////////////Scientific Graphic Calculator////////////////////////////////
+
+File:         matrixwidget.cpp
+Author:       Rainer Strobel
+Email:        rainer1223@users.sourceforge.net
+Homepage:     http://extcalc-linux.sourceforge.net
+
+This program is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 2, or (at your option)
+any later version.
+
+
+////////////////////////////////////////////////////////////////////////////////////////////*/
 #include "matrixwidget.h"
 
 
@@ -7,13 +22,15 @@ void MatrixWidget::resizeEvent(QResizeEvent*)
 	int height=geometry().bottom() - geometry().top();
 
 	
-	varTable->setGeometry(20,50,width/4-30,height-290);
-	outputTable->setGeometry(width/4+10,50,3*width/4-30,height-290);
+//	varTable->setGeometry(20,50,width/4-30,height-290);
+//	outputTable->setGeometry(width/4+10,50,3*width/4-30,height-290);
+	split->setGeometry(20,50,width-40,height-290);
 
-	operationBox->setGeometry(320,height-220,145,35);
-	sprodButton->setGeometry(490,height-220,35,35);
-	invertButton->setGeometry(535,height-220,35,35);
-	detButton->setGeometry(580,height-220,35,35);
+//	operationBox->setGeometry(320,height-220,145,35);
+//	sprodButton->setGeometry(490,height-220,35,35);
+//	invertButton->setGeometry(535,height-220,35,35);
+//	detButton->setGeometry(580,height-220,35,35);
+	dockArea->setGeometry(320,height-220,width-340,35);
 
 	switch(state)
 	{
@@ -694,8 +711,15 @@ void MatrixWidget::typeBoxSlot(int index)
 	}
 }
 
+void MatrixWidget::dockWindowSlot()
+{
+	dockArea->moveDockWindow(toolBar);
+}
 
-
+void MatrixWidget::catalogSlot()
+{
+	catalog->exec(toolBar->mapToGlobal(QPoint(catalogButton->x(),catalogButton->y()+catalogButton->height())));
+}
 
 
 
@@ -754,7 +778,7 @@ void MatrixWidget::calcButtonSlot()
 					results[c]=matrix[size*size+c];
 					for(int c1=c; c1<size; c1++)
 					{
-						if(c!=c)
+						if(c1!=c)
 							results[c]-=results[c1]*matrix[c+c1*size];
 					}
 					results[c]/=matrix[c+c*size];
@@ -895,7 +919,7 @@ void MatrixWidget::calcButtonSlot()
 					else if(size==3)
 					{
 						num=runCalc(input1->text(),&pref,vars);
-						threadData->vars[var][9].fval=Complex(num);
+						threadData->vars[var][0].fval=Complex(num);
 						num=runCalc(input2->text(),&pref,vars);
 						threadData->vars[var][4].fval=Complex(num);
 						num=runCalc(input3->text(),&pref,vars);
