@@ -1,6 +1,22 @@
+/*/////////////////////////////////////////Extcalc////////////////////////////////////////////
+/////////////////////////////////Scientific Graphic Calculator////////////////////////////////
+
+File:         scriptio.cpp
+Author:       Rainer Strobel
+Email:        rainer1223@users.sourceforge.net
+Homepage:     http://extcalc-linux.sourceforge.net
+
+This program is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 2, or (at your option)
+any later version.
+
+
+////////////////////////////////////////////////////////////////////////////////////////////*/
 #include "scriptio.h"
 
 
+
 void ScriptIOWidget::getPref(Preferences newPref)
 {
 	emit prefChange(newPref);
@@ -15,14 +31,16 @@ void ScriptIOWidget::maximizeSlot()
 		maximized=false;
 		calcButtons->show();
 		extButtons->show();
-		maximizeButton->setText(CALCWIDGETC_STR1);
+//		maximizeButton->setText(CALCWIDGETC_STR1);
+		maximizeButton->setIconSet(*maximizeIcon);
 		resizeEvent(NULL);
 	}
 	else 
 	{
 		calcButtons->hide();
 		extButtons->hide();
-		maximizeButton->setText(CALCWIDGETC_STR2);
+//		maximizeButton->setText(CALCWIDGETC_STR2);
+		maximizeButton->setIconSet(*minimizeIcon);
 		maximized=true;
 		resizeEvent(NULL);
 	}
@@ -54,7 +72,7 @@ void ScriptIOWidget::killSlot()
 			if(pref.clearScriptMemory)
 				clearMemSlot();
 		}
-		setTextMode(true);
+		setDisplayMode(SCRIPTTEXT);
 	}
 }
 
@@ -70,7 +88,7 @@ void ScriptIOWidget::clearMemSlot()
 
 void ScriptIOWidget::paintEvent(QPaintEvent*)
 {
-	if(textMode)
+	if(displayType==SCRIPTTEXT)
 	{
 		buffer->fill();
 		QPainter p;
@@ -156,13 +174,23 @@ void ScriptIOWidget::paintEvent(QPaintEvent*)
 	
 		p.end();
 		p.begin(this);
-		p.drawPixmap(20,50,*buffer);
+
+		p.drawPixmap(ioFieldX,ioFieldY,*buffer);
 		p.end();
 	}
-	else {
-		
-
+	else if(displayType==SCRIPT3D)
 		glWindow->repaint(true);
+	else {
+		QPainter p;
+/*		buffer->fill();
+		p.begin(buffer);
+		p.drawText(100,100,"Hallo");
+		p.end();
+*/		p.begin(this);
+
+		p.drawPixmap(ioFieldX,ioFieldY,*buffer);
+
+		p.end();
 	}
 
 }
@@ -175,60 +203,37 @@ void ScriptIOWidget::resizeEvent(QResizeEvent*)
 	int height=geometry().bottom()-geometry().top();
 	
 	
-
-	/*some testing
-	double plbd[3]={-5.0,-5.0,-5.0},prbd[3]={ 5.0,-5.0,-5.0},prfd[3]={ 5.0,-5.0, 5.0},plfd[3]={-5.0,-5.0, 5.0};
-	double plbu[3]={-5.0, 5.0,-5.0},prbu[3]={ 5.0, 5.0,-5.0},prfu[3]={ 5.0, 5.0, 5.0},plfu[3]={-5.0, 5.0, 5.0};
-	
-	glWindow->clearAllLists();
-	int num=glWindow->startList();
-	glWindow->setDrawColor(QColor(255,0,0));
-	glWindow->drawPolygon(plbd,prbd,prbu);
-	glWindow->drawPolygon(plbd,prbu,plbu);
-	glWindow->setDrawColor(QColor(255,255,0));
-	glWindow->drawPolygon(plfd,prfd,prfu);
-	glWindow->drawPolygon(plfd,prfu,plfu);
-	
-	glWindow->setDrawColor(QColor(0,255,0));
-	glWindow->drawPolygon(plfd,plbd,plbu);
-	glWindow->drawPolygon(plfd,plbu,plfu);
-	glWindow->setDrawColor(QColor(255,0,255));
-	glWindow->drawPolygon(prfd,prbd,prbu);
-	glWindow->drawPolygon(prfd,prbu,prfu);
-	
-	glWindow->setDrawColor(QColor(0,0,255));
-	glWindow->drawPolygon(plfd,prfd,prbd);
-	glWindow->drawPolygon(plfd,prbd,plbd);
-	glWindow->setDrawColor(QColor(0,255,255));
-	glWindow->drawPolygon(plfu,prfu,prbu);
-	glWindow->drawPolygon(plfu,prbu,plbu);
-	glWindow->endList();
-	glWindow->enableList(num);
-	
-*/	
-
 	if(maximized)
 	{
-		maximizeButton->setGeometry(10,height-45,90,35);
-		killButton->setGeometry(110,height-45,90,35);
-		runButton->setGeometry(210,height-45,90,35);
+//		maximizeButton->setGeometry(10,height-45,90,35);
+//		killButton->setGeometry(110,height-45,90,35);
+//		runButton->setGeometry(210,height-45,90,35);
+		
+		dockArea->setGeometry(0,menuBottom,width,35);
+		
 		if(inputMode==IMDEFAULT)
-			scrollBar->setGeometry(width-40,50,20,height-100);
+			scrollBar->setGeometry(width-40,menuBottom+40,20,height-100);
 		ioFieldWidth=width-40;
 		ioFieldHeight=height-100;
+		ioFieldX=20;
+		ioFieldY=menuBottom+40;
 	}
 	else {
 		calcButtons->setGeometry(20,height-220,280,200);
 		extButtons->setGeometry(320,height-180,300,160);
-		maximizeButton->setGeometry(325,height-220,90,35);
-		killButton->setGeometry(425,height-220,90,35);
-		runButton->setGeometry(525,height-220,90,35);
+//		maximizeButton->setGeometry(325,height-220,90,35);
+//		killButton->setGeometry(425,height-220,90,35);
+//		runButton->setGeometry(525,height-220,90,35);
+		dockArea->setGeometry(325,height-220,width-345,35);
+
 		if(inputMode==IMDEFAULT)
 			scrollBar->setGeometry(width-40,50,20,height-290);
 		ioFieldWidth=width-40;
 		ioFieldHeight=height-290;
+		ioFieldX=20;
+		ioFieldY=50;
 	}
-	if(!textMode)
+	if(displayType==SCRIPT3D)
 	{
 		int glWidth=ioFieldWidth,glHeight=ioFieldHeight;
 		if(autosize)
@@ -237,8 +242,9 @@ void ScriptIOWidget::resizeEvent(QResizeEvent*)
 				glWidth=glHeight;
 			else glHeight=glWidth;
 		}
-				
-		glWindow->setGeometry(10+(width-20-glWidth)/2,50,glWidth,glHeight);
+
+		glWindow->setGeometry(10+(width-20-glWidth)/2,ioFieldY,glWidth,glHeight);
+
 	}
 	if(inputMode==IMDEFAULT)
 		ioFieldWidth-=20;
@@ -267,6 +273,7 @@ void ScriptIOWidget::resizeEvent(QResizeEvent*)
 	
 
 	buffer->resize(ioFieldWidth,ioFieldHeight);
+//	buffer->fill();
 	if(cursorX>charNum)
 		cursorX=charNum;
 	if(cursorY<scrollBar->value())
@@ -346,7 +353,7 @@ void ScriptIOWidget::keyPressEvent(QKeyEvent*e)
 					cursorY--;
 				if(e->key() == Qt::Key_Down && cursorY<scrollBar->value()+lineNum)
 					cursorY++;
-				repaint(20,50,ioFieldWidth,ioFieldHeight,false);
+				repaint(ioFieldX,ioFieldY,ioFieldWidth,ioFieldHeight,false);
 			}
 			else if(inputMode==IMGETLINE)
 			{
@@ -372,7 +379,7 @@ void ScriptIOWidget::keyPressEvent(QKeyEvent*e)
 						cursorX=charNum-1;
 					}
 				}
-				repaint(20,50,ioFieldWidth,ioFieldHeight,false);
+				repaint(ioFieldX,ioFieldY,ioFieldWidth,ioFieldHeight,false);
 			}
 			break;
 		default:
@@ -425,7 +432,7 @@ void ScriptIOWidget::keyPressEvent(QKeyEvent*e)
 						insert(&inputBuffer[bufferCursor],false);
 						cursorX=cx;
 						cursorY=cy;
-						repaint(20,50,ioFieldWidth,ioFieldHeight,false);
+						repaint(ioFieldX,ioFieldY,ioFieldWidth,ioFieldHeight,false);
 					}
 				}
 				else if(inputMode==IMGETKEY)
@@ -596,7 +603,73 @@ void ScriptIOWidget::customEvent(QCustomEvent*ev)
 				free(ev->data());
 				break;
 			}
-					
+			case SIGDRAW:
+			{
+				if(displayType==SCRIPT2D)
+				{
+					char*data=(char*)ev->data();
+					char command=data[0];
+					QPainter p;
+					p.begin(buffer);
+					p.setPen(*drawPen);
+					int r,g,b,x,y,xs,ys,xe,ye;
+					switch(command)
+					{
+						case 0:
+							buffer->fill();
+							break;
+						case 1:
+							memcpy(&x,&data[1],4);
+							memcpy(&y,&data[5],4);
+							p.drawPoint(x,y);
+							break;
+						case 2:
+							memcpy(&r,&data[1],4);
+							memcpy(&g,&data[5],4);
+							memcpy(&b,&data[9],4);
+							drawPen->setColor(QColor(r,g,b));
+							break;
+						case 3:
+							memcpy(&x,&data[1],4);
+							memcpy(&y,&data[5],4);
+							p.setFont(*drawFont);
+							p.setBackgroundMode(Qt::OpaqueMode);
+							p.drawText(x,y,&data[9]);
+							break;
+						case 4:
+							memcpy(&xs,&data[1],4);
+							memcpy(&ys,&data[5],4);
+							memcpy(&xe,&data[9],4);
+							memcpy(&ye,&data[13],4);
+							p.drawLine(xs,ys,xe,ye);
+							break;
+						case 5:
+							memcpy(&xs,&data[1],4);
+							memcpy(&ys,&data[5],4);
+							memcpy(&xe,&data[9],4);
+							memcpy(&ye,&data[13],4);
+							p.drawRect(xs,ys,xe,ye);
+							break;
+						case 6:
+							memcpy(&xs,&data[1],4);
+							memcpy(&ys,&data[5],4);
+							memcpy(&xe,&data[9],4);
+							memcpy(&ye,&data[13],4);
+							p.drawEllipse(xs,ys,xe,ye);
+							break;
+					}
+					p.end();
+					gettimeofday(&currentTime,NULL);
+					if(currentTime.tv_sec > drawTime.tv_sec || currentTime.tv_usec-drawTime.tv_usec > 20000)
+					{
+						gettimeofday(&drawTime,NULL);
+						if(!t->isActive())
+							t->start(30,true);
+					}
+				}
+				free(ev->data());
+				break;
+			}
 			case SIGCLEARTEXT:
 			{
 				
@@ -701,6 +774,7 @@ void ScriptIOWidget::customEvent(QCustomEvent*ev)
 					else fileData=NULL;
 				}
 				else fileData=NULL;
+				perror("Path: "+path+"\nData: "+QString(fileData));
 				
 				if(fileData==NULL)
 					threadData->data=calloc(2,1);
@@ -772,7 +846,7 @@ void ScriptIOWidget::customEvent(QCustomEvent*ev)
 			}
 			case SIGFINISHED:		//script stopped
 			{
-				setTextMode(true);
+				setDisplayMode(SCRIPTTEXT);
 				gettimeofday(&currentTime,NULL);
 				int secs=currentTime.tv_sec-startTime.tv_sec;
 				int usecs=currentTime.tv_usec-startTime.tv_usec;
@@ -845,12 +919,12 @@ void ScriptIOWidget::mousePressEvent(QMouseEvent*e)
 		return;
 	}
 	int scrollbarPos=scrollBar->value();
-	selectEndLine=selectStartLine=(e->y()-50)/charHeight+scrollbarPos;
-	selectEndRow=selectStartRow=(e->x()-20)/charWidth;
+	selectEndLine=selectStartLine=(e->y()-ioFieldY)/charHeight+scrollbarPos;
+	selectEndRow=selectStartRow=(e->x()-ioFieldX)/charWidth;
 	
 	if(inputMode!=IMDEFAULT)
 		return;
-	int mouseX=e->x()-20,mouseY=e->y()-50;
+	int mouseX=e->x()-ioFieldX,mouseY=e->y()-ioFieldY;
 
 	if(mouseX>=0 && mouseX <=ioFieldWidth && mouseY>=0 && mouseY<=ioFieldHeight)
 	{
@@ -864,25 +938,33 @@ void ScriptIOWidget::mousePressEvent(QMouseEvent*e)
 	
 
 
-	repaint(20,50,ioFieldWidth,ioFieldHeight,false);
+	repaint(ioFieldX,ioFieldY,ioFieldWidth,ioFieldHeight,false);
 }
 void ScriptIOWidget::mouseReleaseEvent(QMouseEvent*me)
 {
 	int scrollbarPos=scrollBar->value();
 //	p.drawText(0,charHeight*(c+1-scrollbarPos),lines[c]);
-	selectEndLine=(me->y()-50)/charHeight+scrollbarPos;
-	selectEndRow=(me->x()-20)/charWidth;
-	repaint(20,50,ioFieldWidth,ioFieldHeight,false);
+	selectEndLine=(me->y()-ioFieldY)/charHeight+scrollbarPos;
+	selectEndRow=(me->x()-ioFieldX)/charWidth;
+	repaint(ioFieldX,ioFieldY,ioFieldWidth,ioFieldHeight,false);
 }
 void ScriptIOWidget::mouseMoveEvent(QMouseEvent*me)
 {
 	int scrollbarPos=scrollBar->value();
 	int oldEndLine=selectEndLine,oldEndRow=selectEndRow;
-	selectEndLine=(me->y()-50)/charHeight+scrollbarPos;
-	selectEndRow=(me->x()-20)/charWidth;
+	selectEndLine=(me->y()-ioFieldY)/charHeight+scrollbarPos;
+	selectEndRow=(me->x()-ioFieldX)/charWidth;
+	if(selectEndRow<0)
+		selectEndRow=0;
+	if(selectEndLine<0)
+		selectEndLine=0;
+	if(selectEndLine<scrollbarPos)
+		scrollBar->setValue(selectEndLine);
+	else if(selectEndLine>scrollbarPos+lineNum)
+		scrollBar->setValue(selectEndLine-lineNum);
 //	perror(QString::number(selectStartLine)+" "+QString::number(selectStartRow)+" "+QString::number(selectEndLine)+" "+QString::number(selectEndRow));
 	if(oldEndRow!=selectEndRow || oldEndLine!=selectEndLine)
-		repaint(20,50,ioFieldWidth,ioFieldHeight,false);
+		repaint(ioFieldX,ioFieldY,ioFieldWidth,ioFieldHeight,false);
 }
 
 
@@ -966,7 +1048,7 @@ void ScriptIOWidget::insert(QString text,bool redraw)
 	}
 	
 	if(redraw)
-		repaint(20,50,ioFieldWidth,ioFieldHeight,false);
+		repaint(ioFieldX,ioFieldY,ioFieldWidth,ioFieldHeight,false);
 
 }
 
@@ -977,14 +1059,14 @@ void ScriptIOWidget::backKey()
 	{
 		lines[cursorY].remove(cursorX-1,1);
 		cursorX--;
-		repaint(20,50,ioFieldWidth,ioFieldHeight,false);
+		repaint(ioFieldX,ioFieldY,ioFieldWidth,ioFieldHeight,false);
 	}
 }
 
 void ScriptIOWidget::deleteKey()
 {
 		lines[cursorY].remove(cursorX,1);
-		repaint(20,50,ioFieldWidth,ioFieldHeight,false);
+		repaint(ioFieldX,ioFieldY,ioFieldWidth,ioFieldHeight,false);
 }
 
 
@@ -997,7 +1079,7 @@ void ScriptIOWidget::clearAll()
 	{
 		lines[c].fill(' ',charNum);
 	}
-	repaint(20,50,ioFieldWidth,ioFieldHeight,false);
+	repaint(ioFieldX,ioFieldY,ioFieldWidth,ioFieldHeight,false);
 }
 
 
@@ -1050,19 +1132,18 @@ void ScriptIOWidget::runScript(QString*code)
 	
 	runningPref=pref;
 	
+	//test
 	searchScripts(code);
 	countDifference=0;
 	initDebugging(code);
-	glModeRequest=false;
-	int ret=preferencesPreprocessor(code,&runningPref);
-
+	runningPref.scriptGraphicsMode=SCRIPTTEXT;
+	
+	char* cleanString=preprocessor(code,&runningPref,true);
+/*	perror("after");
 	if(ret!=0)
 	{
 		switch(ret)
 		{
-			case PPINVALIDCOMMAND:
-				insert("\nPreference Preprocessor: Invalid preprocessor command\n");
-				return;
 			case PPINVALIDARGUMENT:
 				insert("\nPreference Preprocessor: Invalid preference argument\n");
 				return;
@@ -1077,12 +1158,24 @@ void ScriptIOWidget::runScript(QString*code)
 				return;
 		}
 	}
-	if(glModeRequest)
-		glWindow->setPref(runningPref);
+	ret=macroPreprocessor(code);
+	if(ret!=0)
+	{
+		if(ret==PINVALIDCOMMAND)
+			insert("\nMacro Preprocessor: Invalid Preprocessor Command\n");
+		else if(ret==MPINVALIDMACRO)
+			insert("\nMacro Preprocessor: Invalid Macro\n");
+		return;
+	}
+	
+
 
 
 	char*cleanString=checkString(*code,&runningPref);
-
+*/
+	modeRequest=runningPref.scriptGraphicsMode;
+	if(modeRequest==SCRIPT3D)
+		glWindow->setPref(runningPref);
 	delete scriptObject;
 	insert(SCRIPTIO_STR8);
 	if(cleanString==NULL)
@@ -1092,7 +1185,7 @@ void ScriptIOWidget::runScript(QString*code)
 	}
 	scriptObject=new Script(NULL,cleanString,&runningPref,vars,threadData);
 	loadSubScripts();
-	delete[]cleanString;
+	free(cleanString);
 
 //	perror("main thread: "+QString::number(pthread_self()));
 }
@@ -1132,8 +1225,10 @@ void ScriptIOWidget::runSlot()
 
 	killButton->setEnabled(true);
 	scrollBar->hide();
-	if(glModeRequest)
-		setTextMode(false);
+	if(modeRequest==SCRIPT3D)
+		setDisplayMode(SCRIPT3D);
+	else if(modeRequest==SCRIPT2D)
+		setDisplayMode(SCRIPT2D);
 	resizeEvent(NULL);
 
 	gettimeofday(&drawTime,NULL);
@@ -1148,26 +1243,27 @@ void ScriptIOWidget::runSlot()
 
 void ScriptIOWidget::searchScripts(QString*code)
 {
-	char*cleanString=checkString(*code,&pref);
-	if(cleanString==NULL)
+	Preferences tmpPref=pref;
+	char*cleanStr=preprocessor(code,&tmpPref,true);
+	if(cleanStr==NULL)
 		return;
 	char*scriptName;
 	QString filePath;
-	int len=strlen(cleanString);
+	int len=strlen(cleanStr);
 	int pos=0,pos2;
 	FILE*subFile;
 	while(pos<len)
 	{
-		if(strncmp(&cleanString[pos],"run(\"",4) == 0) //run("")
+		if(strncmp(&cleanStr[pos],"run(\"",4) == 0) //run("")
 		{
-			pos2=bracketFind(cleanString,")",pos+4);
-			if(pos2-pos<7 || cleanString[pos2-1]!='\"')
+			pos2=bracketFind(cleanStr,")",pos+4);
+			if(pos2-pos<7 || cleanStr[pos2-1]!='\"')
 			{
 				pos++;
 				continue;
 			}
 			scriptName=new char[pos2-pos-5];
-			strcopy(scriptName,&cleanString[pos+5],pos2-pos-6);
+			strcopy(scriptName,&cleanStr[pos+5],pos2-pos-6);
 			bool newScript=true;
 			for(int c=0; c<threadData->subprogramPath.GetLen(); c++)
 				if(strcmp(threadData->subprogramPath[c],scriptName)==0)
@@ -1204,7 +1300,7 @@ void ScriptIOWidget::searchScripts(QString*code)
 		}
 		else pos++;
 	}
-	delete[] cleanString;
+	delete[] cleanStr;
 }
 
 
@@ -1216,6 +1312,7 @@ void ScriptIOWidget::loadSubScripts()
 	char*cleanSubFileContent;
 	QString qSubFileContent;
 	Script*subScript;
+	Preferences tmpPref=pref;
 	FILE*subFile;
 	for(int c=0; c<threadData->subprogramPath.GetLen(); c++)
 	{
@@ -1235,7 +1332,7 @@ void ScriptIOWidget::loadSubScripts()
 				qSubFileContent=QString(subFileContent);
 				countDifference+=semicolonLines.GetLen();
 				initDebugging(&qSubFileContent);
-				cleanSubFileContent=checkString(qSubFileContent,&pref);
+				cleanSubFileContent=preprocessor(&qSubFileContent,&tmpPref,true);
 				qApp->processEvents();
 				insert(SCRIPTIO_STR9);
 				insert(threadData->subprogramPath[c]);
@@ -1333,12 +1430,17 @@ int ScriptIOWidget::preferencesPreprocessor(QString *code,Preferences*pref)
 					end=code->length();
 				
 				configLine=code->mid(newlinePos+1,end-newlinePos-1);
-				code->remove(newlinePos+1,end-newlinePos-1);
+				
 				pos=newlinePos;
 				configLine=configLine.stripWhiteSpace();
 				if(configLine.find("#config")==0)
 					configLine=configLine.right(configLine.length()-7);
-				else return PPINVALIDCOMMAND;
+				else
+				{
+					pos=newlinePos+2;
+					continue;
+				}
+				code->remove(newlinePos+1,end-newlinePos-1);
 				if((commentPos=configLine.find("//"))!=-1)
 					configLine=configLine.left(commentPos);
 				
@@ -1375,7 +1477,9 @@ int ScriptIOWidget::preferencesPreprocessor(QString *code,Preferences*pref)
 					else return PPINVALIDARGUMENT;
 				}
 				else if(configLine=="gl")
-					glModeRequest=true;
+					modeRequest=SCRIPT3D;
+				else if(configLine=="graphics")
+					modeRequest=SCRIPT2D;
 				else if(configLine=="rasteron")
 					pref->raster=true;
 				else if(configLine=="rasteroff")
@@ -1401,6 +1505,101 @@ int ScriptIOWidget::preferencesPreprocessor(QString *code,Preferences*pref)
 		return PPEMPTY;
 	else return 0;
 }
+int ScriptIOWidget::macroPreprocessor(QString *code)
+{
+	perror("macroPreprocessor: "+(*code));
+	int pos=0,newlinePos=-1,end,commentPos=0,seperatorPos,macroPos;
+	QString macroLine,macro,replacement;
+	bool quote=false;
+	
+	while(pos<(signed)code->length())
+	{
+		perror("outer loop");
+		if(quote)
+		{
+			if((*code)[pos]=='"')
+				quote=false;
+		}
+		else {
+			if((*code)[pos]=='"')
+				quote=true;
+			else if((*code)[pos]=='/' && (*code)[pos+1]=='/')
+			{
+				pos=code->find('\n',pos);
+				if(pos==-1)
+					pos=code->length();
+				else newlinePos=pos;
+			}
+			else if((*code)[pos]=='\n')
+				newlinePos=pos;
+			else if((*code)[pos]=='#')
+			{
+				end=code->find('\n',pos);
+				if(end<0)
+					end=code->length();
+				
+				macroLine=code->mid(newlinePos+1,end-newlinePos-1);
+				
+				pos=newlinePos;
+				macroLine=macroLine.stripWhiteSpace();
+				if(macroLine.find("#define")==0)
+					macroLine=macroLine.right(macroLine.length()-7);
+				else return PINVALIDCOMMAND;
+				if((commentPos=macroLine.find("//"))!=-1)
+					macroLine=macroLine.left(commentPos);
+				
+				code->remove(newlinePos+1,end-newlinePos-1);
+				macroLine=macroLine.stripWhiteSpace();
+
+				
+				seperatorPos=macroLine.find(" ");
+				if(seperatorPos==-1)
+					seperatorPos=macroLine.find("\t");
+				if(seperatorPos>0)
+				{
+					macro=macroLine.left(seperatorPos);
+					replacement=macroLine.right(macroLine.length()-seperatorPos-1);
+					replacement=replacement.stripWhiteSpace();
+					
+					if(!(macro[0]>='a' && macro[0]<='z' ||
+					    macro[0]>='A' && macro[0]<='A' ||
+						macro[0]=='_'))
+						return MPINVALIDMACRO;
+					for(unsigned int c=1; c<macro.length(); c++)
+					{
+						if(!(macro[c]>='a' && macro[c]<='z' ||
+						     macro[c]>='A' && macro[c]<='A' ||
+						     macro[c]>='0' && macro[c]<='9' ||
+							 macro[c]=='_'))
+							return MPINVALIDMACRO;
+					}
+
+					macroPos=0;
+					while((macroPos=code->find(macro,macroPos+1))!=-1)
+					{
+						if(!(macroPos>0 && ((*code)[macroPos-1] >= 'a' && (*code)[macroPos-1]<='z' ||
+											   (*code)[macroPos-1] >= 'A' && (*code)[macroPos-1]<='Z'
+											   || (*code)[macroPos-1] == '_') ||
+											   code->length()>macroPos+macro.length() &&
+											   ((*code)[macroPos+macro.length()] >= 'a' && (*code)[macroPos+macro.length()]<='z' ||
+											   (*code)[macroPos+macro.length()] >= 'A' && (*code)[macroPos+macro.length()]<='Z' ||
+											   (*code)[macroPos+macro.length()] >= '0' && (*code)[macroPos+macro.length()]<='9'
+											   || (*code)[macroPos+macro.length()] == '_')
+							))
+						{
+							code->replace(macroPos,macro.length(),replacement);
+						}
+					}
+
+				}
+			}
+		}
+		pos++;
+	}
+	if(code->length()<=0)
+		return MPEMPTY;
+	else return 0;
+}
 
 void ScriptIOWidget::editSlot(int type)
 {
@@ -1443,7 +1642,7 @@ void ScriptIOWidget::timerSlot()
 		threadData->usleep=true;
 	}
 	*/
-	repaint(20,50,ioFieldWidth,ioFieldHeight,false);
+	repaint(ioFieldX,ioFieldY,ioFieldWidth,ioFieldHeight,false);
 }
 
 
@@ -1515,7 +1714,7 @@ void ScriptIOWidget::contextMenuSlot(int item)
 					memmove(&inputBuffer[bufferCursor+strlen(clipText)],&inputBuffer[bufferCursor],strlen(inputBuffer)-bufferCursor+1);
 					memcpy(&inputBuffer[bufferCursor],clipText,strlen(clipText));
 					bufferCursor+=strlen(clipText);
-					repaint(20,50,ioFieldWidth,ioFieldHeight,false);
+					repaint(ioFieldX,ioFieldY,ioFieldWidth,ioFieldHeight,false);
 				}
 			}
 			break;
@@ -1528,21 +1727,22 @@ void ScriptIOWidget::contextMenuSlot(int item)
 			selectStartLine=0;
 			selectEndLine=lines.GetLen()-1;
 			selectEndRow=lines[selectEndLine].length()-1;
-			repaint(20,50,ioFieldWidth,ioFieldHeight,false);
+			repaint(ioFieldX,ioFieldY,ioFieldWidth,ioFieldHeight,false);
 			break;
 	}
 }
 
-void ScriptIOWidget::setTextMode(bool on)
+void ScriptIOWidget::dockWindowSlot()
 {
-	if(on)
+	dockArea->moveDockWindow(toolBar);
+}
+
+void ScriptIOWidget::setDisplayMode(int mode)
+{
+	displayType=mode;
+
+	if(mode==SCRIPT3D)
 	{
-		textMode=true;
-		glWindow->hide();
-		repaint(true);
-	}
-	else {
-		textMode=false;
 		glWindow->show();
 		glWindow->scrReset();
 		glWindow->resetRotation();
@@ -1550,6 +1750,16 @@ void ScriptIOWidget::setTextMode(bool on)
 		repaint(true);
 		
 	}
+	else {
+		if(mode==SCRIPT2D)
+		{
+			drawPen->setColor(QColor(0,0,0));
+			buffer->fill();
+		}
+		glWindow->hide();
+		repaint(true);
+	}
+	
 }
 
 
@@ -1705,6 +1915,7 @@ void ScriptGL::wheelEvent(QWheelEvent*e)
 
 void ScriptGL::initializeGL()
 {
+	makeCurrent();
 	if(paintActive)
 		glEnd();
 	if(drawListActive)
@@ -1729,6 +1940,7 @@ void ScriptGL::initializeGL()
 	glShadeModel( GL_SMOOTH);
 	glEnable(GL_TEXTURE_2D);
 	glEnable(GL_BLEND);
+	glEnable(GL_DEPTH_TEST);
 	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
 	glDisable(GL_TEXTURE_2D);
 	glMatrixMode( GL_MODELVIEW );
@@ -1747,7 +1959,7 @@ void ScriptGL::resizeGL( int w, int h )
 
 void ScriptGL::paintGL()
 {
-	if(paintActive)
+	if(paintActive || staticListActive)
 		return;
 	if(drawListActive)
 		drawListControl();
